@@ -1,4 +1,4 @@
-/* $Id: address.c,v 1.38 2005/09/26 03:27:59 judd Exp $ */
+/* $Id: address.c,v 1.40 2006/07/21 22:25:24 rikster5 Exp $ */
 
 /*******************************************************************************
  * address.c
@@ -43,7 +43,7 @@ static int glob_sort_rule;
 #define SORT_BY_COMPANY 1
 #define SORT_JAPANESE 2
 #define SORT_JOS 4
-int sort_override=0;
+int sort_by_company=-1;
 
 #ifdef JPILOT_DEBUG
 int print_address_list(AddressList **al)
@@ -97,26 +97,26 @@ int address_compare(const void *v1, const void *v2)
    if (!(glob_sort_rule & SORT_JAPANESE) | (glob_sort_rule & SORT_JOS)) { /* normal */
       if (a1->entry[sort1] || a1->entry[sort2]) {
 	 if (a1->entry[sort1] && a1->entry[sort2]) {
-	    if ((str1 = (char *)malloc(strlen(a1->entry[sort1])+strlen(a1->entry[sort2])+1)) == NULL) {
+	    if ((str1 = malloc(strlen(a1->entry[sort1])+strlen(a1->entry[sort2])+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str1, a1->entry[sort1]);
 	    strcat(str1, a1->entry[sort2]);
 	 }
 	 if (a1->entry[sort1] && (!a1->entry[sort2])) {
-	    if ((str1 = (char *)malloc(strlen(a1->entry[sort1])+1)) == NULL) {
+	    if ((str1 = malloc(strlen(a1->entry[sort1])+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str1, a1->entry[sort1]);
 	 }
 	 if ((!a1->entry[sort1]) && a1->entry[sort2]) {
-	    if ((str1 = (char *)malloc(strlen(a1->entry[sort2])+1)) == NULL) {
+	    if ((str1 = malloc(strlen(a1->entry[sort2])+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str1, a1->entry[sort2]);
 	 }
       } else if (a1->entry[sort3]) {
-	 if ((str1 = (char *)malloc(strlen(a1->entry[sort3])+1)) == NULL) {
+	 if ((str1 = malloc(strlen(a1->entry[sort3])+1)) == NULL) {
 	    return 0;
 	 }
 	 strcpy(str1, a1->entry[sort3]);
@@ -126,26 +126,26 @@ int address_compare(const void *v1, const void *v2)
 
       if (a2->entry[sort1] || a2->entry[sort2]) {
 	 if (a2->entry[sort1] && a2->entry[sort2]) {
-	    if ((str2 = (char *)malloc(strlen(a2->entry[sort1])+strlen(a2->entry[sort2])+1)) == NULL) {
+	    if ((str2 = malloc(strlen(a2->entry[sort1])+strlen(a2->entry[sort2])+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str2, a2->entry[sort1]);
 	    strcat(str2, a2->entry[sort2]);
 	 }
 	 if (a2->entry[sort1] && (!a2->entry[sort2])) {
-	    if ((str2 = (char *)malloc(strlen(a2->entry[sort1])+1)) == NULL) {
+	    if ((str2 = malloc(strlen(a2->entry[sort1])+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str2, a2->entry[sort1]);
 	 }
 	 if ((!a2->entry[sort1]) && a2->entry[sort2]) {
-	    if ((str2 = (char *)malloc(strlen(a2->entry[sort2])+1)) == NULL) {
+	    if ((str2 = malloc(strlen(a2->entry[sort2])+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str2, a2->entry[sort2]);
 	 }
       } else if (a2->entry[sort3]) {
-	 if ((str2 = (char *)malloc(strlen(a2->entry[sort3])+1)) == NULL) {
+	 if ((str2 = malloc(strlen(a2->entry[sort3])+1)) == NULL) {
 	    return 0;
 	 }
 	 strcpy(str2, a2->entry[sort3]);
@@ -159,7 +159,7 @@ int address_compare(const void *v1, const void *v2)
 	 if (a1->entry[sort1] && a1->entry[sort2]) {
 	    if (!(tmp_p1 = strchr(a1->entry[sort1],'\1'))) tmp_p1=a1->entry[sort1]+1;
 	    if (!(tmp_p2 = strchr(a1->entry[sort2],'\1'))) tmp_p2=a1->entry[sort2]+1;
-	    if ((str1 = (char *)malloc(strlen(tmp_p1)+strlen(tmp_p2)+1)) == NULL) {
+	    if ((str1 = malloc(strlen(tmp_p1)+strlen(tmp_p2)+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str1, tmp_p1);
@@ -167,21 +167,21 @@ int address_compare(const void *v1, const void *v2)
 	 }
 	 if (a1->entry[sort1] && (!a1->entry[sort2])) {
 	    if (!(tmp_p1 = strchr(a1->entry[sort1],'\1'))) tmp_p1=a1->entry[sort1]+1;
-	    if ((str1 = (char *)malloc(strlen(tmp_p1)+1)) == NULL) {
+	    if ((str1 = malloc(strlen(tmp_p1)+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str1, tmp_p1);
 	 }
 	 if ((!a1->entry[sort1]) && a1->entry[sort2]) {
 	    if (!(tmp_p2 = strchr(a1->entry[sort2],'\1'))) tmp_p2=a1->entry[sort2]+1;
-	    if ((str1 = (char *)malloc(strlen(tmp_p2)+1)) == NULL) {
+	    if ((str1 = malloc(strlen(tmp_p2)+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str1, tmp_p2);
 	 }
       } else if (a1->entry[sort3]) {
 	 if (!(tmp_p3 = strchr(a1->entry[sort3],'\1'))) tmp_p3=a1->entry[sort3]+1;
-	 if ((str1 = (char *)malloc(strlen(tmp_p3)+1)) == NULL) {
+	 if ((str1 = malloc(strlen(tmp_p3)+1)) == NULL) {
 	    return 0;
 	 }
 	 strcpy(str1, tmp_p3);
@@ -193,7 +193,7 @@ int address_compare(const void *v1, const void *v2)
 	 if (a2->entry[sort1] && a2->entry[sort2]) {
 	    if (!(tmp_p1 = strchr(a2->entry[sort1],'\1'))) tmp_p1=a2->entry[sort1]+1;
 	    if (!(tmp_p2 = strchr(a2->entry[sort2],'\1'))) tmp_p2=a2->entry[sort2]+1;
-	    if ((str2 = (char *)malloc(strlen(tmp_p1)+strlen(tmp_p2)+1)) == NULL) {
+	    if ((str2 = malloc(strlen(tmp_p1)+strlen(tmp_p2)+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str2, tmp_p1);
@@ -201,21 +201,21 @@ int address_compare(const void *v1, const void *v2)
 	 }
 	 if (a2->entry[sort1] && (!a2->entry[sort2])) {
 	    if (!(tmp_p1 = strchr(a2->entry[sort1],'\1'))) tmp_p1=a2->entry[sort1]+1;
-	    if ((str2 = (char *)malloc(strlen(tmp_p1)+1)) == NULL) {
+	    if ((str2 = malloc(strlen(tmp_p1)+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str2, tmp_p1);
 	 }
 	 if ((!a2->entry[sort1]) && a2->entry[sort2]) {
 	    if (!(tmp_p2 = strchr(a2->entry[sort2],'\1'))) tmp_p2=a2->entry[sort2]+1;
-	    if ((str2 = (char *)malloc(strlen(tmp_p2)+1)) == NULL) {
+	    if ((str2 = malloc(strlen(tmp_p2)+1)) == NULL) {
 	       return 0;
 	    }
 	    strcpy(str2, tmp_p2);
 	 }
       } else if (a2->entry[sort3]) {
 	 if (!(tmp_p3 = strchr(a2->entry[sort3],'\1'))) tmp_p3=a2->entry[sort3]+1;
-	 if ((str2 = (char *)malloc(strlen(tmp_p3)+1)) == NULL) {
+	 if ((str2 = malloc(strlen(tmp_p3)+1)) == NULL) {
 	    return 0;
 	 }
 	 strcpy(str2, tmp_p3);
@@ -250,7 +250,6 @@ int address_sort(AddressList **al, int sort_order)
 {
    AddressList *temp_al;
    AddressList **sort_al;
-   struct AddressAppInfo ai;
    int count, i;
    long use_jos, char_set;
 
@@ -264,10 +263,8 @@ int address_sort(AddressList **al, int sort_order)
       return EXIT_SUCCESS;
    }
 
-   get_address_app_info(&ai);
-   glob_sort_rule = ai.sortByCompany;
-   if (sort_override) {
-      glob_sort_rule = !(ai.sortByCompany & SORT_BY_COMPANY);
+   if (sort_by_company) {
+      glob_sort_rule = SORT_BY_COMPANY;
    }
    get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set == CHAR_SET_JAPANESE || char_set == CHAR_SET_SJIS_UTF) {
