@@ -1,4 +1,4 @@
-/* $Id: address_gui.c,v 1.108 2005/10/28 01:20:40 judd Exp $ */
+/* $Id: address_gui.c,v 1.124 2006/07/21 22:25:24 rikster5 Exp $ */
 
 /*******************************************************************************
  * address_gui.c
@@ -93,6 +93,11 @@ char *field_names_ja[]={"kana(Last)", "Last",  "kana(First)", "First",
 #define ADDRESS_MAX_CLIST_NAME 30
 #define ADDRESS_MAX_COLUMN_LEN 80
 
+/* Many RFCs require that the line termination be CRLF rather than just \n.
+ * For conformance to standards this requires adding the two-byte string to
+ * the end of strings destined for export */
+#define CRLF "\x0D\x0A"
+
 GtkWidget *clist;
 GtkWidget *address_text[NUM_ADDRESS_ENTRIES+NUM_ADDRESS_EXT_ENTRIES];
 #ifdef ENABLE_GTK2
@@ -126,10 +131,6 @@ int address_category=CATEGORY_ALL;
 int address_phone_label_selected[NUM_PHONE_ENTRIES];
 static int clist_row_selected;
 extern GtkTooltips *glob_tooltips;
-#ifdef ENABLE_GTK2
-extern gchar *glob_gtk_clipboard_text;
-extern gchar *glob_x11_clipboard_text;
-#endif
 
 static AddressList *glob_address_list=NULL;
 static AddressList *export_address_list=NULL;
@@ -142,7 +143,6 @@ static GtkWidget *undelete_record_button;
 static GtkWidget *copy_record_button;
 static GtkWidget *cancel_record_button;
 static int record_changed;
-static int clist_hack;
 
 static void connect_changed_signals(int con_or_dis);
 static void address_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
@@ -170,8 +170,7 @@ static void init()
    }
 }
 
-static void
-set_new_button_to(int new_state)
+static void set_new_button_to(int new_state)
 {
    jp_logf(JP_LOG_DEBUG, "set_new_button_to new %d old %d\n", new_state, record_changed);
 
@@ -181,65 +180,53 @@ set_new_button_to(int new_state)
 
    switch (new_state) {
     case MODIFY_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
-      clist_hack=TRUE;
-      /* The line selected on the clist becomes unhighlighted, so we do this */
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
-      gtk_widget_show(apply_record_button);
       gtk_widget_show(cancel_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(apply_record_button);
+
+      gtk_widget_hide(add_record_button);
       gtk_widget_hide(delete_record_button);
+      gtk_widget_hide(new_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case NEW_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
-      clist_hack=TRUE;
-      /* The line selected on the clist becomes unhighlighted, so we do this */
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
-      gtk_widget_show(add_record_button);
       gtk_widget_show(cancel_record_button);
+      gtk_widget_show(add_record_button);
+
+      gtk_widget_hide(apply_record_button);
       gtk_widget_hide(copy_record_button);
       gtk_widget_hide(delete_record_button);
+      gtk_widget_hide(new_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case CLEAR_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-      clist_hack=FALSE;
-      gtk_widget_show(new_record_button);
-      gtk_widget_show(copy_record_button);
       gtk_widget_show(delete_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(new_record_button);
+
+      gtk_widget_hide(add_record_button);
+      gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case UNDELETE_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-      clist_hack=FALSE;
-      gtk_widget_hide(delete_record_button);
       gtk_widget_show(undelete_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(new_record_button);
+
+      gtk_widget_hide(add_record_button);
+      gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
+      gtk_widget_hide(delete_record_button);
       break;
+
     default:
       return;
    }
-   switch (record_changed) {
-    case MODIFY_FLAG:
-      gtk_widget_hide(apply_record_button);
-      gtk_widget_hide(cancel_record_button);
-      gtk_widget_show(copy_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-    case NEW_FLAG:
-      gtk_widget_hide(add_record_button);
-      gtk_widget_hide(cancel_record_button);
-      gtk_widget_show(copy_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-    case CLEAR_FLAG:
-      if (new_state != UNDELETE_FLAG)
-      {
-         gtk_widget_hide(new_record_button);
-         gtk_widget_hide(delete_record_button);
-      }
-      break;
-    case UNDELETE_FLAG:
-      gtk_widget_hide(undelete_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-   }
+
    record_changed=new_state;
 }
 
@@ -483,8 +470,7 @@ int address_import_callback(GtkWidget *parent_window, const char *file_path, int
 #ifdef JPILOT_DEBUG
 	 printf("category is [%s]\n", text);
 #endif
-	 strncpy(old_cat_name, text, 16);
-	 old_cat_name[16]='\0';
+	 g_strlcpy(old_cat_name, text, 17);
 	 attrib=0;
 	 /* Figure out what the best category number is */
 	 suggested_cat_num=0;
@@ -555,12 +541,10 @@ int address_import_callback(GtkWidget *parent_window, const char *file_path, int
       for (temp_addrlist=addrlist; temp_addrlist; temp_addrlist=temp_addrlist->next) {
 	 index=temp_addrlist->maddr.unique_id-1;
 	 if (index<0) {
-	    strncpy(old_cat_name, _("Unfiled"), 16);
-	    old_cat_name[16]='\0';
+	    g_strlcpy(old_cat_name, _("Unfiled"), 17);
 	    index=0;
 	 } else {
-	    strncpy(old_cat_name, cai.name[index], 16);
-	    old_cat_name[16]='\0';
+	    g_strlcpy(old_cat_name, cai.name[index], 17);
 	 }
 	 attrib=0;
 	 /* Figure out what category it was in the dat file */
@@ -695,7 +679,7 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
    char pref_time[40];
    int i, r, n, len;
    char *button_text[]={N_("OK")};
-   char *button_overwrite_text[]={N_("Yes"), N_("No")};
+   char *button_overwrite_text[]={N_("No"), N_("Yes")};
    char text[1024];
    char csv_text[65550];
    long char_set, use_jos;
@@ -707,8 +691,7 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
    /* this stuff is for vcard only. */
    /* todo: create a pre-export switch */
    get_pref(PREF_USER, NULL, &svalue);
-   strncpy(text, svalue, sizeof(text));
-   text[sizeof(text)-1]='\0';
+   g_strlcpy(text, svalue, sizeof(text));
    str_to_ical_str(username, sizeof(username), text);
    get_pref(PREF_USER_ID, &userid, NULL);
    gethostname(text, sizeof(text));
@@ -721,15 +704,15 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
       if (S_ISDIR(statb.st_mode)) {
 	 g_snprintf(text, sizeof(text), _("%s is a directory"), filename);
 	 dialog_generic(GTK_WINDOW(export_window),
-			0, 0, _("Error Opening File"),
-			_("Directory"), text, 1, button_text);
+			_("Error Opening File"),
+			DIALOG_ERROR, text, 1, button_text);
 	 return;
       }
       g_snprintf(text, sizeof(text), _("Do you want to overwrite file %s?"), filename);
       r = dialog_generic(GTK_WINDOW(export_window),
-			 0, 0, _("Overwrite File?"),
-			 _("Overwrite File"), text, 2, button_overwrite_text);
-      if (r!=DIALOG_SAID_1) {
+			 _("Overwrite File?"),
+			 DIALOG_QUESTION, text, 2, button_overwrite_text);
+      if (r!=DIALOG_SAID_2) {
 	 return;
       }
    }
@@ -738,8 +721,8 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
    if (!out) {
       g_snprintf(text, sizeof(text), _("Error opening file: %s"), filename);
       dialog_generic(GTK_WINDOW(export_window),
-		     0, 0, _("Error Opening File"),
-		     _("Filename"), text, 1, button_text);
+		     _("Error Opening File"),
+		     DIALOG_ERROR, text, 1, button_text);
       return;
    }
 
@@ -854,16 +837,17 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
 
        case EXPORT_TYPE_VCARD:
 	 /* RFC 2426: vCard MIME Directory Profile */
-	 fprintf(out, "BEGIN:VCARD\nVERSION:3.0\n");
-	 fprintf(out, "PRODID:%s\n", FPI_STRING);
+	 fprintf(out, "BEGIN:VCARD%s", CRLF);
+	 fprintf(out, "VERSION:3.0%s", CRLF);
+	 fprintf(out, "PRODID:%s%s", FPI_STRING, CRLF);
 	 if (maddr->attrib & dlpRecAttrSecret) {
-	    fprintf(out, "CLASS:PRIVATE\n");
+	    fprintf(out, "CLASS:PRIVATE%s", CRLF);
 	 }
-	 fprintf(out, "UID:palm-addressbook-%08x-%08lx-%s@%s\n",
-		 maddr->unique_id, userid, username, hostname);
+	 fprintf(out, "UID:palm-addressbook-%08x-%08lx-%s@%s%s",
+		 maddr->unique_id, userid, username, hostname, CRLF);
 	 str_to_vcard_str(csv_text, sizeof(csv_text),
 			  address_app_info.category.name[maddr->attrib & 0x0F]);
-	 fprintf(out, "CATEGORIES:%s\n", csv_text);
+	 fprintf(out, "CATEGORIES:%s%s", csv_text, CRLF);
 	 if (maddr->addr.entry[0] || maddr->addr.entry[1]) {
 	    char *last = maddr->addr.entry[0];
 	    char *first = maddr->addr.entry[1];
@@ -879,7 +863,7 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	       str_to_vcard_str(csv_text, sizeof(csv_text), last);
 	       fprintf(out, "%s", csv_text);
 	    }
-	    fprintf(out, "\n");
+	    fprintf(out, CRLF);
 	    fprintf(out, "N:");
 	    if (last) {
 	       str_to_vcard_str(csv_text, sizeof(csv_text), last);
@@ -891,32 +875,32 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	       str_to_vcard_str(csv_text, sizeof(csv_text), first);
 	       fprintf(out, "%s", csv_text);
 	    }
-	    fprintf(out, "\n");
+	    fprintf(out, CRLF);
 	 } else if (maddr->addr.entry[2]) {
 	    str_to_vcard_str(csv_text, sizeof(csv_text), maddr->addr.entry[2]);
-	    fprintf(out, "FN:%s\nN:%s\n", csv_text, csv_text);
+	    fprintf(out, "FN:%s%sN:%s%s", csv_text, CRLF, csv_text, CRLF);
 	 } else {
-	    fprintf(out, "FN:-Unknown-\nN:known-;-Un\n");
+	    fprintf(out, "FN:-Unknown-%sN:known-;-Un%s", CRLF, CRLF);
 	 }
 	 if (maddr->addr.entry[13]) {
 	    str_to_vcard_str(csv_text, sizeof(csv_text), maddr->addr.entry[13]);
-	    fprintf(out, "TITLE:%s\n", csv_text);
+	    fprintf(out, "TITLE:%s%s", csv_text, CRLF);
 	 }
 	 if (maddr->addr.entry[2]) {
 	    str_to_vcard_str(csv_text, sizeof(csv_text), maddr->addr.entry[2]);
-	    fprintf(out, "ORG:%s\n", csv_text);
+	    fprintf(out, "ORG:%s%s", csv_text, CRLF);
 	 }
 	 for (n = 3; n < 8; n++) {
 	    if (maddr->addr.entry[n]) {
 	       str_to_vcard_str(csv_text, sizeof(csv_text), maddr->addr.entry[n]);
 	       if (maddr->addr.phoneLabel[n - 3] == 4) {
-		  fprintf(out, "EMAIL:%s\n", csv_text);
+		  fprintf(out, "EMAIL:%s%s", csv_text, CRLF);
 	       } else {
 		  fprintf(out, "TEL;TYPE=%s", vCardMapType(maddr->addr.phoneLabel[n - 3]));
 		  if (maddr->addr.showPhone == n - 3) {
 		     fprintf(out, ",pref");
 		  }
-		  fprintf(out, ":%s\n", csv_text);
+		  fprintf(out, ":%s%s", csv_text, CRLF);
 	       }
 	    }
 	 }
@@ -932,7 +916,7 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
 		  fprintf(out, ";");
 	       }
 	    }
-	    fprintf(out, "\n");
+	    fprintf(out, CRLF);
 	 }
 	 if (maddr->addr.entry[14] || maddr->addr.entry[15] || maddr->addr.entry[16] ||
 	     maddr->addr.entry[17] || maddr->addr.entry[18]) {
@@ -946,15 +930,15 @@ void cb_addr_export_ok(GtkWidget *export_window, GtkWidget *clist,
 		     fprintf(out, " ");
 		  }
 		  if (n==18 && firstnote) {
-		     fprintf(out, "%s\\n\n", csv_text);
+		     fprintf(out, "%s\\n%s", csv_text, CRLF);
 		  } else {
-		     fprintf(out, "%s:\\n\n %s\\n\n", labels[n-14], csv_text);
+		     fprintf(out, "%s:\\n%s %s\\n%s", labels[n-14], CRLF, csv_text, CRLF);
 		  }
 		  firstnote=0;
 	       }
 	    }
 	 }
-	 fprintf(out, "END:VCARD\n");
+	 fprintf(out, "END:VCARD%s", CRLF);
 	 break;
        case EXPORT_TYPE_LDIF:
 	 /* RFC 2256 - organizationalPerson */
@@ -1201,17 +1185,9 @@ static void cb_cancel(GtkWidget *widget, gpointer data)
 void cb_resort(GtkWidget *widget,
 	       gpointer   data)
 {
-   int by_company;
    MyAddress *maddr;
 
-   by_company=GPOINTER_TO_INT(data);
-
-   if (sort_override) {
-      sort_override=0;
-   } else {
-      sort_override=1;
-      by_company=!(by_company & 1);
-   }
+   sort_by_company = !(sort_by_company & 1);
 
    /* Return to this record after resorting */
    maddr = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
@@ -1226,14 +1202,13 @@ void cb_resort(GtkWidget *widget,
    address_find();
 
    /* Update labels after redrawing clist to work around GTK bug */
-   if (by_company) {
+   if (sort_by_company) {
       gtk_clist_set_column_title(GTK_CLIST(clist), ADDRESS_NAME_COLUMN, _("Company/Name"));
    } else {
       gtk_clist_set_column_title(GTK_CLIST(clist), ADDRESS_NAME_COLUMN, _("Name/Company"));
    }
 }
 
-
 void cb_phone_menu(GtkWidget *item, unsigned int value)
 {
    if (!item)
@@ -1351,7 +1326,7 @@ static void cb_add_new_record(GtkWidget *widget,
 	    str1 = gtk_editable_get_chars(GTK_EDITABLE(address_text[i]), 0, -1);
 	    str2 = gtk_editable_get_chars(GTK_EDITABLE(address_text[i+NUM_ADDRESS_ENTRIES]), 0, -1);
 #endif
-	    if ((str0 = (char *)malloc(strlen(str1)+strlen(str2)+2))!=NULL) {
+	    if ((str0 = malloc(strlen(str1)+strlen(str2)+2))!=NULL) {
 	       if (*str1 !='\0') {
 		  strcpy(str0, str1);strcat(str0,"\1");strcat(str0, str2);
 	       } else {
@@ -1488,8 +1463,9 @@ void cb_address_clear(GtkWidget *widget,
 {
    addr_clear_details();
    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
-   gtk_widget_grab_focus(GTK_WIDGET(address_text[0]));
+   connect_changed_signals(DISCONNECT_SIGNALS);
    set_new_button_to(NEW_FLAG);
+   gtk_widget_grab_focus(GTK_WIDGET(address_text[0]));
 }
 
 /* Attempt to make the best possible string out of whatever garbage we find
@@ -1617,7 +1593,7 @@ static void cb_category(GtkWidget *item, int selection)
    if (!item) return;
    if ((GTK_CHECK_MENU_ITEM(item))->active) {
       b=dialog_save_changed_record(pane, record_changed);
-      if (b==DIALOG_SAID_1) {
+      if (b==DIALOG_SAID_2) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
 
@@ -1702,25 +1678,33 @@ static void cb_clist_selection(GtkWidget      *clist,
    struct Address *addr;
    MyAddress *maddr;
    int cat, count, sorted_position;
+   unsigned int unique_id = 0;
    int i, i2;
-   int keep, b;
+   int b;
    char *tmp_p;
    char *clist_text;
    const char *entry_text;
    long use_jos, char_set;
 
-   if ((!event) && (clist_hack)) return;
+   if ((record_changed==MODIFY_FLAG) || (record_changed==NEW_FLAG)) {
+      maddr = gtk_clist_get_row_data(GTK_CLIST(clist), row);
+      if (maddr!=NULL) {
+	 unique_id = maddr->unique_id;
+      }
 
-   /* HACK, see clist hack explanation in memo_gui.c */
-   if (clist_hack) {
-      keep=record_changed;
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, column);
       b=dialog_save_changed_record(pane, record_changed);
-      if (b==DIALOG_SAID_1) {
+      if (b==DIALOG_SAID_2) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
       set_new_button_to(CLEAR_FLAG);
-      clist_select_row(GTK_CLIST(clist), row, column);
+
+      if (unique_id)
+      {
+	 glob_find_id = unique_id;
+         address_find();
+      } else {
+	 clist_select_row(GTK_CLIST(clist), row, column);
+      }
       return;
    }
 
@@ -1738,9 +1722,7 @@ static void cb_clist_selection(GtkWidget      *clist,
       */
    {
       set_new_button_to(UNDELETE_FLAG);
-   }
-   else
-   {
+   } else {
       set_new_button_to(CLEAR_FLAG);
    }
 
@@ -1943,22 +1925,37 @@ static gboolean cb_key_pressed_left_side(GtkWidget   *widget,
                                          GdkEventKey *event,
                                          gpointer     next_widget)
 {
+   GtkWidget *entry_widget;
+#ifdef ENABLE_GTK2
+   GtkTextBuffer *text_buffer;
+   GtkTextIter    iter;
+#endif
+
    if (event->keyval == GDK_Return) {
       gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
 
       switch (gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook))) {
         case 0 : 
-	   gtk_widget_grab_focus(address_text[0]);
+	   entry_widget = address_text[0];
            break;
         case 1 : 
-	   gtk_widget_grab_focus(address_text[8]);
+	   entry_widget = address_text[8];
            break;
         case 2 : 
-	   gtk_widget_grab_focus(address_text[14]);
+	   entry_widget = address_text[14];
            break;
         default:
-	   gtk_widget_grab_focus(address_text[0]);
+	   entry_widget = address_text[0];
       }
+      gtk_widget_grab_focus(entry_widget);
+
+#ifdef ENABLE_GTK2
+      /* Position cursor at start of text */
+      text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(entry_widget));
+      gtk_text_buffer_get_start_iter(text_buffer, &iter);
+      gtk_text_buffer_place_cursor(text_buffer, &iter);
+#endif
+
       return TRUE;
    }
 
@@ -2058,7 +2055,7 @@ static void address_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    get_pixmaps(clist, PIXMAP_NOTE, &pixmap_note, &mask_note);
 
    by_company = address_app_info.sortByCompany;
-   if (sort_override) {
+   if (sort_by_company) {
       by_company=!(by_company & 1);
    }
    if (by_company) {
@@ -2212,7 +2209,7 @@ static void address_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    /* If there are items in the list, highlight the selected row */
    if ((main) && (entries_shown>0)) {
       /* Select the existing requested row, or row 0 if that is impossible */
-      if (clist_row_selected <= entries_shown)
+      if (clist_row_selected < entries_shown)
       {
 	 clist_select_row(GTK_CLIST(clist), clist_row_selected, ADDRESS_PHONE_COLUMN);
 	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), clist_row_selected)) {
@@ -2319,7 +2316,7 @@ int address_cycle_cat()
    int i, new_cat;
 
    b=dialog_save_changed_record(pane, record_changed);
-   if (b==DIALOG_SAID_1) {
+   if (b==DIALOG_SAID_2) {
       cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
    }
 
@@ -2488,15 +2485,11 @@ int address_gui_cleanup()
    int b;
 
    b=dialog_save_changed_record(pane, record_changed);
-   if (b==DIALOG_SAID_1) {
+   if (b==DIALOG_SAID_2) {
       cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
    }
    free_AddressList(&glob_address_list);
    connect_changed_signals(DISCONNECT_SIGNALS);
-#ifdef ENABLE_GTK2
-   glob_gtk_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
-   glob_x11_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY));
-#endif
 #ifdef ENABLE_GTK2
    set_pref(PREF_ADDRESS_PANE, gtk_paned_get_position(GTK_PANED(pane)), NULL, TRUE);
 #else
@@ -2569,24 +2562,6 @@ int address_gui(GtkWidget *vbox, GtkWidget *hbox)
       address_category=CATEGORY_ALL;
    }
 
-#ifdef ENABLE_GTK2
-   /* Preserve clipboard buffer across applications */
-   if (glob_gtk_clipboard_text != NULL)
-   {
-      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD),
-                             glob_gtk_clipboard_text, -1);
-      g_free(glob_gtk_clipboard_text);
-      glob_gtk_clipboard_text = NULL;
-   }
-   if (glob_x11_clipboard_text != NULL)
-   {
-      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
-                             glob_x11_clipboard_text, -1);
-      g_free(glob_x11_clipboard_text);
-      glob_x11_clipboard_text = NULL;
-   }
-#endif
-
    accel_group = gtk_accel_group_new();
    gtk_window_add_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(vbox)),
       accel_group);
@@ -2610,7 +2585,7 @@ int address_gui(GtkWidget *vbox, GtkWidget *hbox)
    glob_date_label = gtk_label_new(" ");
    gtk_box_pack_start(GTK_BOX(vbox1), glob_date_label, FALSE, FALSE, 0);
    timeout_date(NULL);
-   glob_date_timer_tag = gtk_timeout_add(CLOCK_TICK, timeout_date, NULL);
+   glob_date_timer_tag = gtk_timeout_add(get_timeout_interval(), timeout_date, NULL);
 
    /* Separator */
    separator = gtk_hseparator_new();
@@ -2641,18 +2616,22 @@ int address_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(vbox1), scrolled_window, TRUE, TRUE, 0);
 
    clist = gtk_clist_new_with_titles(3, titles);
-   clist_hack=FALSE;
+   
    gtk_clist_column_title_passive(GTK_CLIST(clist), ADDRESS_PHONE_COLUMN);
    gtk_clist_column_title_passive(GTK_CLIST(clist), ADDRESS_NOTE_COLUMN);
 
-   if (address_app_info.sortByCompany) {
+   /* Initialize sort_by_company the first time program is called */ 
+   if (sort_by_company == -1) {
+      sort_by_company = address_app_info.sortByCompany;
+   }
+
+   if (sort_by_company) {
       gtk_clist_set_column_title(GTK_CLIST(clist), ADDRESS_NAME_COLUMN, _("Company/Name"));
    } else {
       gtk_clist_set_column_title(GTK_CLIST(clist), ADDRESS_NAME_COLUMN, _("Name/Company"));
    }
    gtk_signal_connect(GTK_OBJECT(GTK_CLIST(clist)->column[ADDRESS_NAME_COLUMN].button),
-		      "clicked", GTK_SIGNAL_FUNC(cb_resort),
-		      GINT_TO_POINTER(address_app_info.sortByCompany));
+		      "clicked", GTK_SIGNAL_FUNC(cb_resort), NULL);
    gtk_clist_set_column_title(GTK_CLIST(clist), ADDRESS_PHONE_COLUMN, _("Phone"));
    /* Put pretty pictures in the clist column headings */
    get_pixmaps(vbox, PIXMAP_NOTE, &pixmap, &mask);
@@ -2687,7 +2666,7 @@ int address_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(hbox_temp), address_quickfind_entry, TRUE, TRUE, 0);
 
    /* The new entry gui */
-   hbox_temp = gtk_hbox_new(FALSE, 0);
+   hbox_temp = gtk_hbox_new(FALSE, 3);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
    /* Create Cancel button */
