@@ -1,6 +1,9 @@
-/* category.c
+/* $Id: category.c,v 1.28 2005/12/18 14:54:39 rousseau Exp $ */
+
+/*******************************************************************************
+ * category.c
  * A module of J-Pilot http://jpilot.org
- * 
+ *
  * Copyright (C) 2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -26,6 +29,7 @@
 #include <sys/types.h>
 #include <utime.h>
 #include <unistd.h>
+#include <stdio.h>
 #include <pi-file.h>
 #include "utils.h"
 #include "log.h"
@@ -42,7 +46,7 @@
 #define EDIT_CAT_ENTRY_CANCEL 105
 
 struct dialog_cats_data {
-   int button_hit;  
+   int button_hit;
    int selected;
    int state;
    GtkWidget *clist;
@@ -71,6 +75,8 @@ int jp_count_records_in_cat(char *db_name, int cat_index)
    count = 0;
 
    num = jp_read_DB_files(db_name, &records);
+   if (-1 == num)
+     return 0;
 
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
@@ -112,14 +118,22 @@ int pdb_file_change_indexes(char *DB_name, int old_index, int new_index)
    char local_pdb_file[FILENAME_MAX];
    char full_local_pdb_file[FILENAME_MAX];
    char full_local_pdb_file2[FILENAME_MAX];
+#ifdef PILOT_LINK_0_12
+   pi_file_t *pf1, *pf2;
+#else
    struct pi_file *pf1, *pf2;
+#endif
    struct DBInfo infop;
    void *app_info;
    void *sort_info;
    void *record;
    int r;
    int idx;
+#ifdef PILOT_LINK_0_12
+   size_t size;
+#else
    int size;
+#endif
    int attr;
    int cat, new_cat;
    int count;
@@ -142,20 +156,20 @@ int pdb_file_change_indexes(char *DB_name, int old_index, int new_index)
 
    pf1 = pi_file_open(full_local_pdb_file);
    if (!pf1) {
-      jp_logf(JP_LOG_WARN, "Couldn't open [%s]\n", full_local_pdb_file);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), full_local_pdb_file);
+      return EXIT_FAILURE;
    }
    pi_file_get_info(pf1, &infop);
    pf2 = pi_file_create(full_local_pdb_file2, &infop);
    if (!pf2) {
-      jp_logf(JP_LOG_WARN, "Couldn't open [%s]\n", full_local_pdb_file2);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), full_local_pdb_file2);
+      return EXIT_FAILURE;
    }
 
    pi_file_get_app_info(pf1, &app_info, &size);
    pi_file_set_app_info(pf2, app_info, size);
 
-   pi_file_get_sort_info(pf1, &sort_info, &size);  
+   pi_file_get_sort_info(pf1, &sort_info, &size);
    pi_file_set_sort_info(pf2, sort_info, size);
 
    count = 0;
@@ -175,12 +189,12 @@ int pdb_file_change_indexes(char *DB_name, int old_index, int new_index)
    pi_file_close(pf2);
 
    if (rename(full_local_pdb_file2, full_local_pdb_file) < 0) {
-      jp_logf(JP_LOG_WARN, "change_indexes: rename failed\n");
+      jp_logf(JP_LOG_WARN, "pdb_file_change_indexes(): %s\n, ", _("rename failed"));
    }
 
    utime(full_local_pdb_file, &times);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int edit_cats_delete_cats_pc3(char *DB_name, int cat)
@@ -196,8 +210,8 @@ int edit_cats_delete_cats_pc3(char *DB_name, int cat)
 
    pc_in = jp_open_home_file(local_pc_file, "r+");
    if (pc_in==NULL) {
-      jp_logf(JP_LOG_WARN, _("Unable to open %s\n"), local_pc_file);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), local_pc_file);
+      return EXIT_FAILURE;
    }
 
    while(!feof(pc_in)) {
@@ -214,14 +228,14 @@ int edit_cats_delete_cats_pc3(char *DB_name, int cat)
       if (rec_len > 0x10000) {
 	 jp_logf(JP_LOG_WARN, _("PC file corrupt?\n"));
 	 fclose(pc_in);
-	 return -1;
+	 return EXIT_FAILURE;
       }
       if (((header.rt==NEW_PC_REC) || (header.rt==REPLACEMENT_PALM_REC)) &&
 	  ((header.attrib&0x0F)==cat)) {
 	 if (fseek(pc_in, -(header.header_len), SEEK_CUR)) {
 	    jp_logf(JP_LOG_WARN, _("fseek failed - fatal error\n"));
 	    fclose(pc_in);
-	    return -1;
+	    return EXIT_FAILURE;
 	 }
 	 count++;
 	 header.rt=DELETED_PC_REC;
@@ -231,7 +245,7 @@ int edit_cats_delete_cats_pc3(char *DB_name, int cat)
       if (fseek(pc_in, rec_len, SEEK_CUR)) {
 	 jp_logf(JP_LOG_WARN, _("fseek failed - fatal error\n"));
 	 fclose(pc_in);
-	 return -1;
+	 return EXIT_FAILURE;
       }
    }
 
@@ -253,8 +267,8 @@ int _edit_cats_change_cats_pc3(char *DB_name, int old_cat,
 
    pc_in = jp_open_home_file(local_pc_file, "r+");
    if (pc_in==NULL) {
-      jp_logf(JP_LOG_WARN, _("Unable to open %s\n"), local_pc_file);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), local_pc_file);
+      return EXIT_FAILURE;
    }
 
    while(!feof(pc_in)) {
@@ -271,14 +285,14 @@ int _edit_cats_change_cats_pc3(char *DB_name, int old_cat,
       if (rec_len > 0x10000) {
 	 jp_logf(JP_LOG_WARN, _("PC file corrupt?\n"));
 	 fclose(pc_in);
-	 return -1;
+	 return EXIT_FAILURE;
       }
       /* No matter what the record type we will change the cat if needed */
       if ((header.attrib&0x0F)==old_cat) {
 	 if (fseek(pc_in, -(header.header_len), SEEK_CUR)) {
 	    jp_logf(JP_LOG_WARN, _("fseek failed - fatal error\n"));
 	    fclose(pc_in);
-	    return -1;
+	    return EXIT_FAILURE;
 	 }
 	 count++;
 	 header.attrib=(header.attrib&0xFFFFFFF0) | new_cat;
@@ -289,7 +303,7 @@ int _edit_cats_change_cats_pc3(char *DB_name, int old_cat,
 	 if (fseek(pc_in, -(header.header_len), SEEK_CUR)) {
 	    jp_logf(JP_LOG_WARN, _("fseek failed - fatal error\n"));
 	    fclose(pc_in);
-	    return -1;
+	    return EXIT_FAILURE;
 	 }
 	 count++;
 	 header.attrib=(header.attrib&0xFFFFFFF0) | old_cat;
@@ -299,7 +313,7 @@ int _edit_cats_change_cats_pc3(char *DB_name, int old_cat,
       if (fseek(pc_in, rec_len, SEEK_CUR)) {
 	 jp_logf(JP_LOG_WARN, _("fseek failed - fatal error\n"));
 	 fclose(pc_in);
-	 return -1;
+	 return EXIT_FAILURE;
       }
    }
 
@@ -333,12 +347,14 @@ int edit_cats_change_cats_pdb(char *DB_name, int old_cat, int new_cat)
    GList *temp_list;
 
    jp_logf(JP_LOG_DEBUG, "edit_cats_change_cats_pdb\n");
-#ifdef EDIT_CATS_DEBUG 
+#ifdef EDIT_CATS_DEBUG
   printf("edit_cats_change_cats_pdb\n");
 #endif
 
    count=0;
    r = jp_read_DB_files(DB_name, &records);
+   if (-1 == r)
+     return 0;
 
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
@@ -398,11 +414,11 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
    int id;
    int button;
    int catnum; /* JPA */
-   unsigned char currentname[HOSTCATLTH]; /* current category name */
-   unsigned char previousname[HOSTCATLTH]; /* previous category name */
-   unsigned char pilotentry[HOSTCATLTH /* yes! */]; /* entry text, in Pilot character set */
-   char *button_text[]={gettext_noop("OK")};
-   char *move_text[]={gettext_noop("Move"), gettext_noop("Delete"), gettext_noop("Cancel")};
+   char currentname[HOSTCATLTH]; /* current category name */
+   char previousname[HOSTCATLTH]; /* previous category name */
+   char pilotentry[HOSTCATLTH /* yes! */]; /* entry text, in Pilot character set */
+   char *button_text[]={N_("OK")};
+   char *move_text[]={N_("Move"), N_("Delete"), N_("Cancel")};
    char *text;
    const char *entry_text;
    char temp[256];
@@ -430,8 +446,8 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	    }
 	 }
 	 if (count>14) {
-	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-			   _("Edit Categories"), NULL,
+	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+			   _("Edit Categories"), DIALOG_ERROR,
 			   _("The maximum number of categories (16) are already used"), 1, button_text);
 	    return;
 	 }
@@ -444,15 +460,15 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	 break;
        case EDIT_CAT_RENAME:
 	 if ((catnum<0) || (Pdata->cai2.name[catnum][0]=='\0')) {
-	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-			   _("Edit Categories"), NULL,
+	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+			   _("Edit Categories Error"), DIALOG_ERROR,
 			   _("You must select a category to rename"), 1, button_text);
 	    return;
 	 }
  	 if (catnum == 0) {
 	    g_snprintf(temp, sizeof(temp), _("You can't edit category %s.\n"), Pdata->cai1.name[0]);
-	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-			   _("Edit Categories"), NULL,
+	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+			   _("Edit Categories Error"), DIALOG_ERROR,
 			   temp, 1, button_text);
 	    return;
 	 }
@@ -469,15 +485,15 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	 printf("delete cat\n");
 #endif
 	 if (catnum<0) {
-	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-			   _("Edit Categories"), NULL,
+	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+			   _("Edit Categories Error"), DIALOG_ERROR,
 			   _("You must select a category to delete"), 1, button_text);
 	    return;
 	 }
 	 if (catnum==0) {
 	    sprintf(temp, _("You can't delete category %s.\n"), Pdata->cai1.name[0]);
-	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-			   _("Edit Categories"), NULL,
+	    dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+			   _("Edit Categories"), DIALOG_ERROR,
 			   temp, 1, button_text);
 	    return;
 	 }
@@ -494,8 +510,8 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	    g_snprintf(temp, sizeof(temp), _("There are %d records in %s.\n"
 		       "Do you want to move them to %s, or delete them?"),
 		       count, Pdata->cai1.name[catnum], Pdata->cai1.name[0]);
-	    r = dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-			       _("Edit Categories"), NULL,
+	    r = dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+			       _("Edit Categories"), DIALOG_QUESTION,
 			       temp, 3, move_text);
 	    switch (r) {
 	     case DIALOG_SAID_1:
@@ -550,7 +566,7 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	 break;
        case EDIT_CAT_ENTRY_OK:
 	 if ((Pdata->state!=EDIT_CAT_RENAME) && (Pdata->state!=EDIT_CAT_NEW)) {
-	    jp_logf(JP_LOG_WARN, "invalid state file %s line %d\n", __FILE__, __LINE__);
+	    jp_logf(JP_LOG_WARN, _("invalid state file %s line %d\n"), __FILE__, __LINE__);
 	    return;
 	 }
 	 /* Can't make an empty category, could do a dialog telling user */
@@ -565,8 +581,7 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	    entry_text = gtk_entry_get_text(GTK_ENTRY(Pdata->entry));
 	    /* JPA convert entry to Pilot character set before checking */
 	    /* note : don't know Pilot size until converted */
-	    strncpy(pilotentry, entry_text, HOSTCATLTH);
-	    pilotentry[HOSTCATLTH-1] = '\0';
+	    g_strlcpy(pilotentry, entry_text, HOSTCATLTH);
 	    charset_j2p(pilotentry, HOSTCATLTH, char_set);
 	    pilotentry[PILOTCATLTH-1] = '\0';
 	    for (i=0; i<CATCOUNT; i++) {
@@ -574,8 +589,8 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	       if (((i != catnum) || (Pdata->state != EDIT_CAT_RENAME))
 		   && !strcmp(pilotentry, Pdata->cai2.name[i])) {
 		  sprintf(temp, _("The category %s can't be used more than once"), entry_text);
-		  dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 0, 0,
-				 _("Edit Categories"), NULL,
+		  dialog_generic(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+				 _("Edit Categories"), DIALOG_ERROR,
 				 temp, 1, button_text);
 		  return;
 	       }
@@ -595,11 +610,10 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	    /* JPA assuming gtk makes a copy */
 	    gtk_clist_set_text(GTK_CLIST(Pdata->clist), i, 0, entry_text);
 	    /* JPA enter new category name in Palm Pilot character set */
-	    charset_j2p((unsigned char*)entry_text, HOSTCATLTH, char_set);
-	    strncpy(Pdata->cai2.name[catnum], entry_text, PILOTCATLTH);
-	    Pdata->cai2.name[catnum][PILOTCATLTH-1]='\0';
+	    charset_j2p((char *)entry_text, HOSTCATLTH, char_set);
+	    g_strlcpy(Pdata->cai2.name[catnum], entry_text, PILOTCATLTH);
 	 }
-	 
+
 	 if (Pdata->state==EDIT_CAT_NEW) {
 #ifdef EDIT_CATS_DEBUG
 	    printf("new cat\n");
@@ -623,8 +637,8 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 		  /* JPA get the old text from listbox, to avoid making */
 		  /* character set conversions */
 	          r = gtk_clist_get_text(GTK_CLIST(Pdata->clist), i, 0, &text);
-                  if (r) strncpy(currentname, text, HOSTCATLTH);
-                  currentname[HOSTCATLTH-1] = '\0';
+                  if (r)
+		     g_strlcpy(currentname, text, HOSTCATLTH);
 		  Pdata->cai2.ID[i]=id;
 		  strcpy(Pdata->cai2.name[i], pilotentry);
 		  Pdata->cai2.renamed[i]=1;
@@ -636,8 +650,8 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
                      if (j < CATCOUNT) {
                         strcpy(previousname, currentname);
 			r = gtk_clist_get_text(GTK_CLIST(Pdata->clist), i, 0, &text);
-                        if (r) strncpy(currentname, text, HOSTCATLTH);
-                        currentname[HOSTCATLTH-1] = '\0';
+                        if (r)
+			   g_strlcpy(currentname, text, HOSTCATLTH);
 		        gtk_clist_set_text(GTK_CLIST(Pdata->clist), i, 0, previousname);
                         j++;
 		     }
@@ -657,7 +671,7 @@ static void cb_edit_button(GtkWidget *widget, gpointer data)
 	 Pdata->state=EDIT_CAT_START;
 	 break;
        default:
-	 jp_logf(JP_LOG_WARN, "cb_edit_button: unknown button\n");
+	 jp_logf(JP_LOG_WARN, "cb_edit_button(): %s\n", "unknown button");
       }
    }
 }
@@ -681,7 +695,7 @@ void cb_clist_edit_cats(GtkWidget *widget,
       printf("cb_clist_edit_cats not in start state\n");
 #endif
       if (Pdata->selected!=row) {
-	 gtk_clist_select_row(GTK_CLIST(Pdata->clist), Pdata->selected, 0);
+	 clist_select_row(GTK_CLIST(Pdata->clist), Pdata->selected, 0);
       }
    }
 }
@@ -724,7 +738,7 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
 {
    GtkWidget *button;
    GtkWidget *hbox, *vbox;
-   GtkWidget *vbox1, *vbox2;
+   GtkWidget *vbox1, *vbox2, *vbox3;
    GtkWidget *dialog;
    GtkWidget *clist;
    GtkWidget *entry;
@@ -734,16 +748,15 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
    int i;
    int j;
    long char_set;
-   unsigned char *catname; /* JPA category names in host character set */
-   char *titles[]={"category name"};
+   char *catname; /* JPA category names in host character set */
+   char *titles[2];
    gchar *empty_line[] = {""};
 
    jp_logf(JP_LOG_DEBUG, "edit_cats\n");
 
    Pdata.selected=-1;
    Pdata.state=EDIT_CAT_START;
-   strncpy(Pdata.db_name, db_name, 16);
-   Pdata.db_name[15]='\0';
+   g_strlcpy(Pdata.db_name, db_name, 16);
 #ifdef EDIT_CATS_DEBUG
    for (i = 0; i < CATCOUNT; i++) {
       if (cai->name[i][0] != '\0') {
@@ -782,11 +795,13 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
 
    gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(gtk_widget_get_toplevel(widget)));
 
+   vbox3 = gtk_vbox_new(FALSE, 0);
    hbox = gtk_hbox_new(FALSE, 0);
 
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 
-   gtk_container_add(GTK_CONTAINER(dialog), hbox);
+   gtk_container_add(GTK_CONTAINER(dialog), vbox3);
+   gtk_container_add(GTK_CONTAINER(vbox3), hbox);
 
    vbox1 = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 1);
@@ -794,9 +809,14 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 1);
 
+   /* correctly translate title */
+   titles[0]=strdup(_("category name"));
+   titles[1]=NULL;
    clist = gtk_clist_new_with_titles(1, titles);
+   if (titles[0]) free(titles[0]);
 
    gtk_clist_column_titles_passive(GTK_CLIST(clist));
+   gtk_clist_set_column_min_width(GTK_CLIST(clist), 0, 100);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
 
    Pdata.clist = clist;
@@ -806,10 +826,17 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
    gtk_box_pack_start(GTK_BOX(vbox1), clist, TRUE, TRUE, 1);
 
    /* Buttons */
-   hbox = gtk_hbox_new(FALSE, 0);
+   hbox = gtk_hbutton_box_new();
+   gtk_container_set_border_width(GTK_CONTAINER(hbox), 12);
+   gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
+   gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 6);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 1);
 
+#ifdef ENABLE_STOCK_BUTTONS
+   button = gtk_button_new_from_stock(GTK_STOCK_NEW);
+#else
    button = gtk_button_new_with_label(_("New"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_edit_button),
 		      GINT_TO_POINTER(EDIT_CAT_NEW));
@@ -821,7 +848,11 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
 		      GINT_TO_POINTER(EDIT_CAT_RENAME));
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 1);
 
+#ifdef ENABLE_STOCK_BUTTONS
+   button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
+#else
    button = gtk_button_new_with_label(_("Delete"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_edit_button),
 		      GINT_TO_POINTER(EDIT_CAT_DELETE));
@@ -836,29 +867,44 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
    gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
    label = gtk_label_new("");
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-   hbox = gtk_hbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
    separator = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
 
    Pdata.label = label;
 
-   entry = gtk_entry_new_with_max_length(HOSTCATLTH);
+   entry = gtk_entry_new_with_max_length(HOSTCATLTH-1);
    gtk_signal_connect(GTK_OBJECT(entry), "activate",
 		      GTK_SIGNAL_FUNC(cb_edit_button),
 		      GINT_TO_POINTER(EDIT_CAT_ENTRY_OK));
-   gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+   hbox = gtk_hbutton_box_new();
+   gtk_container_set_border_width(GTK_CONTAINER(hbox), 12);
+   gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
+   gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 6);
+   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
+   button = gtk_button_new_with_label(_("Cancel"));
+#endif
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_edit_button),
+		      GINT_TO_POINTER(EDIT_CAT_ENTRY_CANCEL));
+   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 1);
+
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_OK);
+#else
    button = gtk_button_new_with_label(_("OK"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_edit_button),
 		      GINT_TO_POINTER(EDIT_CAT_ENTRY_OK));
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 1);
 
-   button = gtk_button_new_with_label(_("Cancel"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_edit_button),
-		      GINT_TO_POINTER(EDIT_CAT_ENTRY_CANCEL));
-   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 1);
+   separator = gtk_hseparator_new();
+   gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
 
    Pdata.entry_box = vbox;
    Pdata.entry = entry;
@@ -876,30 +922,40 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
       }
       if (j < CATCOUNT) {
 	 /* gtk_clist_set_text(GTK_CLIST(clist), i, 0, cai->name[i]); JPA */
-          catname = charset_p2newj((unsigned char*)cai->name[j], PILOTCATLTH, char_set);
+          catname = charset_p2newj(cai->name[j], PILOTCATLTH, char_set);
 	  gtk_clist_set_text(GTK_CLIST(clist), i, 0, catname); /* JPA */
           free(catname);
       }
    }
 
    /* Button Box */
-   hbox = gtk_hbox_new(TRUE, 2);
-   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
-   gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 2);
+   hbox = gtk_hbutton_box_new();
+   gtk_container_set_border_width(GTK_CONTAINER(hbox), 12);
+   gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
+   gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 6);
+   gtk_box_pack_start(GTK_BOX(vbox3), hbox, FALSE, FALSE, 2);
 
    /* Buttons */
-   button = gtk_button_new_with_label(_("OK"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_dialog_button),
-		      GINT_TO_POINTER(DIALOG_SAID_1));
-   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 1);
-
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
    button = gtk_button_new_with_label(_("Cancel"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_dialog_button),
 		      GINT_TO_POINTER(DIALOG_SAID_2));
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 1);
 
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_OK);
+#else
+   button = gtk_button_new_with_label(_("OK"));
+#endif
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_dialog_button),
+		      GINT_TO_POINTER(DIALOG_SAID_1));
+   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 1);
+
 #ifdef EDIT_CATS_DEBUG
    button = gtk_button_new_with_label(_("debug"));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
@@ -920,7 +976,7 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
    gtk_widget_hide(Pdata.entry_box);
 
    gtk_main();
-   
+
    /* OK, we're back */
 #ifdef EDIT_CATS_DEBUG
    if (Pdata.button_hit==DIALOG_SAID_1) {
@@ -943,5 +999,5 @@ int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
 
    memcpy(cai, &(Pdata.cai2), sizeof(struct CategoryAppInfo));
 
-   return 0;
+   return EXIT_SUCCESS;
 }
