@@ -1,6 +1,9 @@
-/* dialer.c
+/* $Id: dialer.c,v 1.1.1.8 2005-12-18 19:01:58 rousseau Exp $ */
+
+/*******************************************************************************
+ * dialer.c
  * A module of J-Pilot http://jpilot.org
- * 
+ *
  * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -88,7 +91,7 @@ static gboolean cb_destroy_dialog(GtkWidget *widget)
 static void set_prefix_label(struct dialog_data *Pdata)
 {
    char str[70];
-   
+
    g_snprintf(str, sizeof(str), "%s%s%s",
 	      GTK_TOGGLE_BUTTON(Pdata->check_pre1)->active ?
 	      gtk_entry_get_text(GTK_ENTRY(Pdata->entry_pre1)) : "",
@@ -114,7 +117,7 @@ static void dialer(gpointer data, int phone_or_ext)
    const char *pref_command;
    char c1, c2;
    int i, len;
-   
+
    Pdata=data;
    if (phone_or_ext==CHOOSE_PHONE) {
       g_snprintf(str, sizeof(str), "%s%s%s%s",
@@ -129,10 +132,8 @@ static void dialer(gpointer data, int phone_or_ext)
    if (phone_or_ext==CHOOSE_EXT) {
       Pext=gtk_entry_get_text(GTK_ENTRY(Pdata->entry_ext));
       if (!Pext) Pext=null_str;
-      strncpy(str, Pext, sizeof(str));
+      g_strlcpy(str, Pext, sizeof(str));
    }
-   str[sizeof(str)-1]='\0';
-
 
    pref_command = gtk_entry_get_text(GTK_ENTRY(Pdata->entry_command));
    /* Make a system call command string */
@@ -159,7 +160,7 @@ static void dialer(gpointer data, int phone_or_ext)
    }
    command[1022]='\0';
 
-   jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "executing command = [%s]\n", command);
+   jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, _("executing command = [%s]\n"), command);
    system(command);
 }
 
@@ -354,12 +355,17 @@ int dialog_dial(GtkWindow *main_window, char *string, char *ext)
 
 
    /* Button Box */
-   hbox1 = gtk_hbox_new(TRUE, 2);
-   gtk_container_set_border_width(GTK_CONTAINER(hbox1), 5);
+   hbox1 = gtk_hbutton_box_new();
+   gtk_container_set_border_width(GTK_CONTAINER(hbox1), 7);
+   gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox1), GTK_BUTTONBOX_END);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 2);
 
    /* Buttons */
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+#else
    button = gtk_button_new_with_label(_("Dismiss"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_dialog_button),
 		      GINT_TO_POINTER(DIALOG_SAID_1));
@@ -386,5 +392,5 @@ int dialog_dial(GtkWindow *main_window, char *string, char *ext)
 
    free(Pdata);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
