@@ -1,6 +1,9 @@
-/* export_gui.c
+/* $Id: export_gui.c,v 1.19 2005/12/18 14:54:39 rousseau Exp $ */
+
+/*******************************************************************************
+ * export_gui.c
  * A module of J-Pilot http://jpilot.org
- * 
+ *
  * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -95,8 +98,7 @@ int export_browse(GtkWidget *main_window, int pref_export)
 
    if (pref_export) {
       get_pref(pref_export, NULL, &svalue);
-      strncpy(dir, svalue, MAX_PREF_VALUE);
-      dir[MAX_PREF_VALUE]='\0';
+      g_strlcpy(dir, svalue, sizeof(dir));
       i=strlen(dir)-1;
       if (i<0) i=0;
       if (dir[i]!='/') {
@@ -330,19 +332,30 @@ int export_gui(GtkWidget *main_window,
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_export_browse), export_window);
 
-   hbox = gtk_hbox_new(FALSE, 0);
+   hbox = gtk_hbutton_box_new();
+   gtk_container_set_border_width(GTK_CONTAINER(hbox), 12);
+   gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
+   gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 6);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-   button = gtk_button_new_with_label(_("OK"));
-   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_ok), export_window);
-
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
    button = gtk_button_new_with_label(_("Cancel"));
+#endif
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_export_quit), export_window);
 
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_OK);
+#else
+   button = gtk_button_new_with_label(_("OK"));
+#endif
+   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_ok), export_window);
+
    if (glob_cb_export_menu) {
       glob_cb_export_menu(export_clist, export_category);
    }
@@ -354,7 +367,7 @@ int export_gui(GtkWidget *main_window,
 
    gtk_main();
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 /*
  * End Export code
