@@ -1,4 +1,4 @@
-/* $Id: jpilot.c,v 1.123 2005/10/24 19:15:41 judd Exp $ */
+/* $Id: jpilot.c,v 1.140 2006/06/09 20:42:57 judd Exp $ */
 
 /*******************************************************************************
  * jpilot.c
@@ -35,6 +35,9 @@
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
 #include <sys/stat.h>
 
 #include <pi-version.h>
@@ -98,12 +101,6 @@ GtkWidget *g_hbox2, *g_vbox0_1;
 
 GtkWidget *glob_date_label;
 GtkTooltips *glob_tooltips;
-#ifdef ENABLE_GTK2
-/* GTK+ clipboard (Ctrl-V/Ctrl-C) */
-gchar *glob_gtk_clipboard_text = NULL;
-/* X11 clipboard (middle mouse click) */
-gchar *glob_x11_clipboard_text = NULL;
-#endif
 gint glob_date_timer_tag;
 pid_t glob_child_pid;
 pid_t jpilot_master_pid;
@@ -121,6 +118,7 @@ GtkWidget *glob_dialog=NULL;
 unsigned char skip_plugins;
 static GtkWidget *button_locked, *button_locked_masked, *button_unlocked, *button_sync, *button_backup;
 GtkCheckMenuItem *menu_hide_privates;
+int t_fmt_ampm = TRUE;
 
 int pipe_from_child, pipe_to_parent;
 int pipe_from_parent, pipe_to_child;
@@ -382,8 +380,7 @@ void call_plugin_help(int number)
 	       plugin->plugin_help(&text, &width, &height);
 	       if (text) {
 		  dialog_generic(GTK_WINDOW(window),
-				width, height,
-				 _("Help"), plugin->name, text, 1, button_text);
+				 plugin->help_name, DIALOG_INFO, text, 1, button_text);
 		  free(text);
 	       }
 	    }
@@ -446,8 +443,8 @@ void cb_print(GtkWidget *widget, gpointer data)
       }
    }
 #endif
-   dialog_generic(GTK_WINDOW(window), 0, 0,
-		  _("Print"), NULL,
+   dialog_generic(GTK_WINDOW(window),
+		  _("Print"), DIALOG_WARNING,
 		  _("There is no print support for this conduit."),
 		  1, button_text);
 }
@@ -506,8 +503,8 @@ void cb_import(GtkWidget *widget, gpointer data)
       }
    }
 #endif
-   dialog_generic(GTK_WINDOW(window), 0, 0,
-		  _("Import"), NULL,
+   dialog_generic(GTK_WINDOW(window),
+		  _("Import"), DIALOG_WARNING,
 		  _("There is no import support for this conduit."),
 		  1, button_text);
 }
@@ -550,8 +547,8 @@ void cb_export(GtkWidget *widget, gpointer data)
       }
    }
 #endif
-   dialog_generic(GTK_WINDOW(window), 0, 0,
-		  _("Export"), NULL,
+   dialog_generic(GTK_WINDOW(window),
+		  _("Export"), DIALOG_WARNING,
 		  _("There is no export support for this conduit."),
 		  1, button_text);
 }
@@ -594,12 +591,12 @@ static void cb_private(GtkWidget *widget, gpointer data)
 				       ascii_password, retry);
 	    r_pass = verify_password(ascii_password);
 	    retry=TRUE;
-	 } while ((r_pass==FALSE) && (r_dialog==1));
+	 } while ((r_pass==FALSE) && (r_dialog==2));
       }
 #else
-      r_dialog = 1;
+      r_dialog = 2;
 #endif
-      if (r_dialog==1) {
+      if (r_dialog==2) {
 	 gtk_widget_hide(button_locked);
 	 gtk_widget_hide(button_locked_masked);
 	 gtk_widget_show(button_unlocked);
@@ -751,13 +748,11 @@ int bad_sync_exit_status(int exit_status)
    if ((exit_status == SYNC_ERROR_NOT_SAME_USERID) ||
        (exit_status == SYNC_ERROR_NOT_SAME_USER)) {
       return dialog_generic(GTK_WINDOW(window),
-			      0, 0,
-			      _("Sync Problem"), _("Sync"), text1, 2, button_text);
+			      _("Sync Problem"), DIALOG_WARNING, text1, 2, button_text);
    }
    if (exit_status == SYNC_ERROR_NULL_USERID) {
       return dialog_generic(GTK_WINDOW(window),
-			    0, 0,
-			    _("Sync Problem"), _("Sync"), text2, 1, button_text);
+			    _("Sync Problem"), DIALOG_ERROR, text2, 1, button_text);
    }
    return EXIT_FAILURE;
 }
@@ -767,24 +762,9 @@ void output_to_pane(const char *str)
    int w, h, new_y;
    long ivalue;
 
-#ifdef ENABLE_GTK2
-   GtkTextIter end_iter;
-   gtk_text_buffer_get_end_iter(g_output_text_buffer, &end_iter);
-
-   if (! g_utf8_validate(str, -1, NULL)) {
-      gchar *utf8_text;
-
-      utf8_text = g_locale_to_utf8 (str, -1, NULL, NULL, NULL);
-      gtk_text_buffer_insert_at_cursor(g_output_text_buffer, utf8_text, -1);
-      g_free(utf8_text);
-   } else
-      gtk_text_buffer_insert(g_output_text_buffer, &end_iter, str, -1);
-   gtk_text_view_scroll_mark_onscreen(g_output_text, gtk_text_buffer_get_insert(g_output_text_buffer));
-#else
-   gtk_text_insert(g_output_text, NULL, NULL, NULL, str, -1);
-#endif
+   /* Adjust window height to user preference or minimum size */
    get_pref(PREF_OUTPUT_HEIGHT, &ivalue, NULL);
-   /* Make them look at least something if output happens */
+   /* Make them look at something if output happens */
    if (ivalue < 60) {
       ivalue=60;
       set_pref(PREF_OUTPUT_HEIGHT, 60, NULL, TRUE);
@@ -792,6 +772,51 @@ void output_to_pane(const char *str)
    gdk_window_get_size(window->window, &w, &h);
    new_y = h - ivalue;
    gtk_paned_set_position(GTK_PANED(output_pane), new_y);
+
+   /* Output text to window */
+#ifdef ENABLE_GTK2
+   GtkTextIter end_iter;
+   GtkTextMark *end_mark;
+   gboolean scroll_to_end = FALSE;
+   gdouble sbar_value, sbar_page_size, sbar_upper;
+ 
+   /* The window position scrolls with new input if the user has left
+    * the scrollbar at the bottom of the window.  Otherwise, if the user
+    * is scrolling back through the log then jpilot does nothing and defers
+    * to the user. */
+
+   /* Get position of scrollbar */
+   sbar_value = g_output_text->vadjustment->value;
+   sbar_page_size = g_output_text->vadjustment->page_size;
+   sbar_upper = g_output_text->vadjustment->upper;
+   /* Keep scrolling to the end only if we are already near(1 window) of the end
+    * OR the window has just been created and is blank */
+   if (abs((sbar_value+sbar_page_size) - sbar_upper) < sbar_page_size  ||
+       sbar_page_size == 1) {
+      scroll_to_end = TRUE;
+   }
+
+   gtk_text_buffer_get_end_iter(g_output_text_buffer, &end_iter);
+
+   if (! g_utf8_validate(str, -1, NULL)) {
+      gchar *utf8_text;
+
+      utf8_text = g_locale_to_utf8 (str, -1, NULL, NULL, NULL);
+      gtk_text_buffer_insert(g_output_text_buffer, &end_iter, utf8_text, -1);
+      g_free(utf8_text);
+   } else
+      gtk_text_buffer_insert(g_output_text_buffer, &end_iter, str, -1);
+
+   if (scroll_to_end) {
+      end_mark = gtk_text_buffer_create_mark(g_output_text_buffer, NULL, &end_iter, TRUE);
+      gtk_text_buffer_move_mark(g_output_text_buffer, end_mark, &end_iter);
+      gtk_text_view_scroll_to_mark(g_output_text, end_mark, 0, TRUE, 0.0, 0.0);
+      gtk_text_buffer_delete_mark(g_output_text_buffer, end_mark);
+   }
+#else
+   /* Nothing fancy is done for GTK1 */
+   gtk_text_insert(g_output_text, NULL, NULL, NULL, str, -1);
+#endif
 }
 
 /* #define PIPE_DEBUG */
@@ -891,8 +916,7 @@ static void cb_read_pipe_from_child(gpointer data,
 		  if (user_len > MAX_PREF_VALUE) {
 		     user_len = MAX_PREF_VALUE;
 		  }
-		  strncpy(user, Pstr2, user_len);
-		  user[user_len] = '\0';
+		  g_strlcpy(user, Pstr2, user_len+1);
 		  jp_logf(JP_LOG_DEBUG, "pipe_read: user = %s\n", user);
 		  set_pref(PREF_USER, 0, user, TRUE);
 	       }
@@ -909,8 +933,7 @@ static void cb_read_pipe_from_child(gpointer data,
 		  if (password_len > MAX_PREF_VALUE) {
 		     password_len = MAX_PREF_VALUE;
 		  }
-		  strncpy(password, Pstr2, password_len);
-		  password[password_len] = '\0';
+		  g_strlcpy(password, Pstr2, password_len+1);
 		  jp_logf(JP_LOG_DEBUG, "pipe_read: password = %s\n", password);
 		  set_pref(PREF_PASSWORD, 0, password, TRUE);
 	       }
@@ -982,9 +1005,8 @@ void cb_about(GtkWidget *widget, gpointer data)
    get_compile_options(options, sizeof(options));
 
    if (GTK_IS_WINDOW(window)) {
-      dialog_generic_with_text(GTK_WINDOW(window),
-			       w, h,
-			       about, PN" "VERSION, options, 1, button_text, 1);
+      dialog_generic(GTK_WINDOW(window),
+			       about, DIALOG_INFO, options, 1, button_text);
    }
 }
 
@@ -1074,6 +1096,41 @@ void cb_install_gui(GtkWidget *widget, gpointer data)
    install_gui_and_size(window);
 }
 
+#ifdef ENABLE_GTK2
+#include <gdk-pixbuf/gdk-pixdata.h>
+
+guint8 *get_inline_pixbuf_data(const char **xpm_icon_data, gint icon_size)
+{
+   GdkPixbuf  *pixbuf;
+   GdkPixdata *pixdata;
+   GdkPixbuf  *scaled_pb;
+   guint8     *data;
+   guint       len;
+
+   pixbuf = gdk_pixbuf_new_from_xpm_data(xpm_icon_data);
+   if (!pixbuf)
+      return NULL;
+
+   if (gdk_pixbuf_get_width(pixbuf)  != icon_size ||
+       gdk_pixbuf_get_height(pixbuf) != icon_size) {
+      scaled_pb = gdk_pixbuf_scale_simple(pixbuf, icon_size, icon_size,
+	 GDK_INTERP_BILINEAR);
+      g_object_unref(G_OBJECT(pixbuf));
+      pixbuf = scaled_pb;
+   }
+
+   pixdata = (GdkPixdata*)g_malloc(sizeof(GdkPixdata));
+   gdk_pixdata_from_pixbuf(pixdata, pixbuf, FALSE);
+   data = gdk_pixdata_serialize(pixdata, &len);
+
+   g_object_unref(pixbuf);
+   g_free(pixdata);
+
+   return data;
+}
+#endif
+
+
 void get_main_menu(GtkWidget  *window,
 		   GtkWidget **menubar,
 		   GList *plugin_list)
@@ -1087,9 +1144,6 @@ void get_main_menu(GtkWidget  *window,
 #define ICON_XPM(icon, size) NULL
 #endif
 
-#ifdef ENABLE_GTK2
-#include <gdk-pixbuf/gdk-pixdata.h>
-
 const char *todo_menu_icon[]={
    "14 14 38 1",
    "# c None",
@@ -1347,36 +1401,6 @@ const char *user_icon[] = {
 "++@@/(_:'<[]}}|]",
 "++@@/^1:2<[345|]"};
 
-guint8 *get_inline_pixbuf_data(const char **xpm_icon_data, gint icon_size)
-{
-   GdkPixbuf  *pixbuf;
-   GdkPixdata *pixdata;
-   GdkPixbuf  *scaled_pb;
-   guint8     *data;
-   guint       len;
-
-   pixbuf = gdk_pixbuf_new_from_xpm_data(xpm_icon_data);
-   if (!pixbuf)
-      return NULL;
-
-   if (gdk_pixbuf_get_width(pixbuf)  != icon_size ||
-       gdk_pixbuf_get_height(pixbuf) != icon_size) {
-      scaled_pb = gdk_pixbuf_scale_simple(pixbuf, icon_size, icon_size,
-	 GDK_INTERP_BILINEAR);
-      g_object_unref(G_OBJECT(pixbuf));
-      pixbuf = scaled_pb;
-   }
-
-   pixdata = (GdkPixdata*)g_malloc(sizeof(GdkPixdata));
-   gdk_pixdata_from_pixbuf(pixdata, pixbuf, FALSE);
-   data = gdk_pixdata_serialize(pixdata, &len);
-
-   g_free(pixbuf);
-   g_free(pixdata);
-
-   return data;
-}
-#endif
 
   GtkItemFactoryEntry menu_items1[]={
   { _("/_File"),                           NULL,         NULL,           0,                  "<Branch>" },
@@ -1436,7 +1460,7 @@ guint8 *get_inline_pixbuf_data(const char **xpm_icon_data, gint icon_size)
 #endif
   { _("/_Help"),                           NULL,         NULL,           0,                  "<LastBranch>" },
   { _("/Help/PayBack program"),            NULL,         cb_payback,     0,                  NULL },
-  { _("/Help/J-Pilot"),                    NULL,         cb_about,       0,                  ICON(GTK_STOCK_DIALOG_INFO) },
+  { _("/Help/About J-Pilot"),              NULL,         cb_about,       0,                  ICON(GTK_STOCK_DIALOG_INFO) },
   { "END",                                 NULL,         NULL,           0,                  NULL }
  };
 
@@ -1865,29 +1889,22 @@ void jp_window_iconify(GtkWidget *window)
 
 void cb_payback(GtkWidget *widget, gpointer data)
 {
-   int w, h;
    char *button_text[]={N_("OK")};
    char *text=
-     "Buy a Palm Tungsten, or Palm Zire, register it on-line and "
+     "Buy a Palm Tungsten, or Palm Zire, register it on-line and\n"
      "earn points for the J-Pilot project.\n\n"
-     "If you already own a Tungsten, or Zire Palm, consider registering it "
-     "on-line to help J-Pilot.  "
+     "If you already own a Tungsten, or Zire Palm, consider registering it\n"
+     "on-line to help J-Pilot.\n"
      "Visit http://jpilot.org/payback.html for details.\n\n"
-     "This message will not be automatically displayed again and can be found later "
+     "This message will not be automatically displayed again and can be found later\n"
      "in the help menu.\n\n\n"
      "PALM, TUNGSTEN and ZIRE are among the trademarks of palmOne, Inc."
      ;
 
-   gdk_window_get_size(window->window, &w, &h);
-
-#ifdef ENABLE_GTK2
-   h = 2;
-#endif
-
    if (GTK_IS_WINDOW(window)) {
-      dialog_generic_with_text(GTK_WINDOW(window), w/2, h*2/3,
-			       PN, "Register your Palm in the Payback Program",
-			       text, 1, button_text, 1);
+      dialog_generic(GTK_WINDOW(window),
+	    "Register your Palm in the Payback Program", DIALOG_INFO,
+	    text, 1, button_text);
    }
 }
 
@@ -1925,7 +1942,9 @@ int main(int argc, char *argv[])
    GtkWidget *temp_vbox;
    GtkWidget *button_datebook,*button_address,*button_todo,*button_memo;
    GtkWidget *button;
+#ifndef ENABLE_GTK2
    GtkWidget *arrow;
+#endif
    GtkWidget *separator;
    GtkStyle *style;
    GdkBitmap *mask;
@@ -2135,6 +2154,11 @@ char * xpm_backup[] = {
    glob_log_gui_mask = JP_LOG_FATAL | JP_LOG_WARN | JP_LOG_GUI;
    glob_find_id = 0;
 
+   /* Directory ~/.jpilot is created with permissions of 700 to prevent anyone
+    * but the user from looking at potentially sensitive files.
+    * Files within the directory have permission 600 */
+   umask(0077);
+
    /* enable internationalization(i18n) before printing any output */
 #if defined(ENABLE_NLS)
 #  ifdef HAVE_LOCALE_H
@@ -2145,16 +2169,30 @@ char * xpm_backup[] = {
 #endif
 
    pref_init();
+
+   /* Determine whether locale supports am/pm time formats */
+#  ifdef HAVE_LANGINFO_H
+      t_fmt_ampm = strcmp(nl_langinfo(T_FMT_AMPM), "");
+#  endif
+
    /* read jpilot.rc file for preferences */
    pref_read_rc_file();
 
    /* Extract first day of week preference from locale in GTK2 */
-#ifdef ENABLE_GTK2
-#  if defined(ENABLE_NLS)
-      week_start = dgettext("gtk20", "calendar:week_start:0");
-      if (strncmp(week_start, "calendar:week_start:", 20) == 0)
-	 pref_fdow = *(week_start + 20) - '0';
-
+#  ifdef ENABLE_GTK2
+#     ifdef HAVE_LANGINFO_H
+	 /* GTK 2.8 libraries */
+	 week_start = nl_langinfo (_NL_TIME_FIRST_WEEKDAY);
+	 pref_fdow = *((unsigned char *) week_start) - 1;
+#     else
+	 /* GTK 2.6 libraries */
+#        if defined(ENABLE_NLS)
+	    week_start = dgettext("gtk20", "calendar:week_start:0");
+	    if (strncmp("calendar:week_start:", week_start, 20) == 0) {
+	       pref_fdow = *(week_start + 20) - '0';
+	    }
+#        endif
+#     endif
       if (pref_fdow > 1)
 	 pref_fdow = 1;
       if (pref_fdow < 0)
@@ -2162,12 +2200,12 @@ char * xpm_backup[] = {
 
       set_pref_possibility(PREF_FDOW, pref_fdow, TRUE);
 #  endif
-#endif
+
 
 #ifdef ENABLE_GTK2
    if (otherconv_init()) {
       printf("Error: could not set encoding\n");
-      return 1;
+      return EXIT_FAILURE;
    }
 #endif
 
@@ -2471,7 +2509,8 @@ char * xpm_backup[] = {
    gtk_container_set_border_width(GTK_CONTAINER(temp_hbox), 5);
    gtk_paned_pack2(GTK_PANED(output_pane), temp_hbox, FALSE, FALSE);
 
-   temp_vbox = gtk_vbox_new(FALSE, 0);
+   temp_vbox = gtk_vbox_new(FALSE, 3);
+   gtk_container_set_border_width(GTK_CONTAINER(temp_vbox), 6);
    gtk_box_pack_end(GTK_BOX(temp_hbox), temp_vbox, FALSE, FALSE, 0);
 
 #ifdef ENABLE_GTK2
@@ -2496,20 +2535,28 @@ char * xpm_backup[] = {
    gtk_box_pack_start(GTK_BOX(temp_hbox), vscrollbar, FALSE, FALSE, 0);
 #endif
 
-   /* button = gtk_button_new_with_label(_("Minimize")); */
-   button = gtk_button_new();
-   arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-   gtk_container_add(GTK_CONTAINER(button), arrow);
-   gtk_box_pack_start(GTK_BOX(temp_vbox), button, TRUE, TRUE, 3);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_output),
-		      GINT_TO_POINTER(OUTPUT_MINIMIZE));
-
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
+#else
    button = gtk_button_new_with_label(_("Clear"));
+#endif
    gtk_box_pack_start(GTK_BOX(temp_vbox), button, TRUE, TRUE, 3);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_output),
 		      GINT_TO_POINTER(OUTPUT_CLEAR));
+
+#ifdef ENABLE_GTK2
+   button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
+#else
+   /* button = gtk_button_new_with_label(_("Minimize")); */
+   button = gtk_button_new();
+   arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
+   gtk_container_add(GTK_CONTAINER(button), arrow);
+#endif
+   gtk_box_pack_start(GTK_BOX(temp_vbox), button, TRUE, TRUE, 3);
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_output),
+		      GINT_TO_POINTER(OUTPUT_MINIMIZE));
    /* End output text */
 
    /* Create "Datebook" button */
@@ -2609,7 +2656,7 @@ char * xpm_backup[] = {
    gtk_signal_connect(GTK_OBJECT(button_sync), "clicked",
 		      GTK_SIGNAL_FUNC(cb_sync),
 		      GINT_TO_POINTER(skip_plugins ? SYNC_NO_PLUGINS : 0));
-   gtk_box_pack_start(GTK_BOX(g_vbox0), button_sync, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(g_vbox0), button_sync, FALSE, FALSE, 3);
 
    gtk_tooltips_set_tip(glob_tooltips, button_sync, _("Sync your palm to the desktop   Ctrl-Y"), NULL);
    gtk_widget_add_accelerator(button_sync, "clicked", accel_group, GDK_y,
@@ -2630,7 +2677,7 @@ char * xpm_backup[] = {
 		      GINT_TO_POINTER
 		      (skip_plugins ? SYNC_NO_PLUGINS | SYNC_FULL_BACKUP
 		      : SYNC_FULL_BACKUP));
-   gtk_box_pack_start(GTK_BOX(g_vbox0), button_backup, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(g_vbox0), button_backup, FALSE, FALSE, 3);
 
    gtk_tooltips_set_tip(glob_tooltips, button_backup, _("Sync your palm to the desktop\n"
 			"and then do a backup"), NULL);
@@ -2826,8 +2873,8 @@ char * xpm_backup[] = {
       g_strlcat(text, _("\" and change the \""), sizeof(text));
       g_strlcat(text, _("Character Set "), sizeof(text));
       g_strlcat(text, _("\"."), sizeof(text));
-      ret = dialog_generic(GTK_WINDOW(window), 0, 0,
-         _("Select a UTF-8 encoding"), NULL, text, 3, button_text);
+      ret = dialog_generic(GTK_WINDOW(window),
+         _("Select a UTF-8 encoding"), DIALOG_QUESTION, text, 3, button_text);
 
       switch (ret) {
        case DIALOG_SAID_1: 
