@@ -1,4 +1,4 @@
-/* $Id: memo_gui.c,v 1.85 2005/10/24 19:15:41 judd Exp $ */
+/* $Id: memo_gui.c,v 1.98 2006/06/25 03:36:36 rikster5 Exp $ */
 
 /*******************************************************************************
  * memo_gui.c
@@ -49,10 +49,6 @@
 #define DISCONNECT_SIGNALS 401
 
 extern GtkTooltips *glob_tooltips;
-#ifdef ENABLE_GTK2
-extern gchar *glob_gtk_clipboard_text;
-extern gchar *glob_x11_clipboard_text;
-#endif
 
 struct MemoAppInfo memo_app_info;
 static int memo_category = CATEGORY_ALL;
@@ -78,7 +74,6 @@ static GtkWidget *undelete_record_button;
 static GtkWidget *copy_record_button;
 static GtkWidget *cancel_record_button;
 static int record_changed;
-static int clist_hack;
 
 static MemoList *glob_memo_list=NULL;
 static MemoList *export_memo_list=NULL;
@@ -92,76 +87,63 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 			      MemoList **memo_list, int category, int main);
 static void cb_add_new_record(GtkWidget *widget, gpointer data);
 
-static void
-set_new_button_to(int new_state)
+static void set_new_button_to(int new_state)
 {
    jp_logf(JP_LOG_DEBUG, "set_new_button_to new %d old %d\n", new_state, record_changed);
+
    if (record_changed==new_state) {
       return;
    }
 
    switch (new_state) {
     case MODIFY_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
-      clist_hack=TRUE;
-      /* The line selected on the clist becomes unhighlighted, so we do this */
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
-      gtk_widget_show(apply_record_button);
       gtk_widget_show(cancel_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(apply_record_button);
+
+      gtk_widget_hide(add_record_button);
       gtk_widget_hide(delete_record_button);
+      gtk_widget_hide(new_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case NEW_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
-      clist_hack=TRUE;
-      /* The line selected on the clist becomes unhighlighted, so we do this */
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
-      gtk_widget_show(add_record_button);
       gtk_widget_show(cancel_record_button);
+      gtk_widget_show(add_record_button);
+
+      gtk_widget_hide(apply_record_button);
       gtk_widget_hide(copy_record_button);
       gtk_widget_hide(delete_record_button);
+      gtk_widget_hide(new_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case CLEAR_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-      clist_hack=FALSE;
-      gtk_widget_show(new_record_button);
-      gtk_widget_show(copy_record_button);
       gtk_widget_show(delete_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(new_record_button);
+
+      gtk_widget_hide(add_record_button);
+      gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case UNDELETE_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-      clist_hack=FALSE;
       gtk_widget_show(undelete_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(new_record_button);
+
+      gtk_widget_hide(add_record_button);
+      gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
       gtk_widget_hide(delete_record_button);
       break;
 
     default:
       return;
    }
-   switch (record_changed) {
-    case MODIFY_FLAG:
-      gtk_widget_hide(apply_record_button);
-      gtk_widget_hide(cancel_record_button);
-      gtk_widget_show(copy_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-    case NEW_FLAG:
-      gtk_widget_hide(add_record_button);
-      gtk_widget_hide(cancel_record_button);
-      gtk_widget_show(copy_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-    case CLEAR_FLAG:
-      if (new_state != UNDELETE_FLAG)
-      {
-         gtk_widget_hide(new_record_button);
-         gtk_widget_hide(delete_record_button);
-      }
-      break;
-    case UNDELETE_FLAG:
-      gtk_widget_hide(undelete_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-   }
+
    record_changed=new_state;
 }
 
@@ -362,8 +344,7 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 #ifdef JPILOT_DEBUG
 	 printf("category is [%s]\n", text);
 #endif
-	 strncpy(old_cat_name, text, 16);
-	 old_cat_name[16]='\0';
+	 g_strlcpy(old_cat_name, text, 17);
 	 attrib=0;
 	 /* Figure out what the best category number is */
 	 suggested_cat_num=0;
@@ -433,12 +414,10 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 	 new_memo.text=temp_memolist->mmemo.memo.text;
 	 index=temp_memolist->mmemo.unique_id-1;
 	 if (index<0) {
-	    strncpy(old_cat_name, _("Unfiled"), 16);
-	    old_cat_name[16]='\0';
+	    g_strlcpy(old_cat_name, _("Unfiled"), 17);
 	    index=0;
 	 } else {
-	    strncpy(old_cat_name, cai.name[index], 16);
-	    old_cat_name[16]='\0';
+	    g_strlcpy(old_cat_name, cai.name[index], 17);
 	 }
 	 attrib=0;
 	 /* Figure out what category it was in the dat file */
@@ -524,7 +503,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    time_t ltime;
    struct tm *now;
    char *button_text[]={N_("OK")};
-   char *button_overwrite_text[]={N_("Yes"), N_("No")};
+   char *button_overwrite_text[]={N_("No"), N_("Yes")};
    char text[1024];
    char str1[256], str2[256];
    char pref_time[40];
@@ -536,15 +515,15 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
       if (S_ISDIR(statb.st_mode)) {
 	 g_snprintf(text, sizeof(text), _("%s is a directory"), filename);
 	 dialog_generic(GTK_WINDOW(export_window),
-			0, 0, _("Error Opening File"),
-			_("Directory"), text, 1, button_text);
+			_("Error Opening File"),
+			DIALOG_ERROR, text, 1, button_text);
 	 return;
       }
       g_snprintf(text,sizeof(text), _("Do you want to overwrite file %s?"), filename);
       r = dialog_generic(GTK_WINDOW(export_window),
-			 0, 0, _("Overwrite File?"),
-			 _("Overwrite File"), text, 2, button_overwrite_text);
-      if (r!=DIALOG_SAID_1) {
+			 _("Overwrite File?"),
+			 DIALOG_ERROR, text, 2, button_overwrite_text);
+      if (r!=DIALOG_SAID_2) {
 	 return;
       }
    }
@@ -553,8 +532,8 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    if (!out) {
       g_snprintf(text,sizeof(text), _("Error opening file: %s"), filename);
       dialog_generic(GTK_WINDOW(export_window),
-		     0, 0, _("Error Opening File"),
-		     _("Filename"), text, 1, button_text);
+		     _("Error Opening File"),
+		     DIALOG_ERROR, text, 1, button_text);
       return;
    }
 
@@ -774,7 +753,7 @@ static void cb_category(GtkWidget *item, int selection)
 
    if ((GTK_CHECK_MENU_ITEM(item))->active) {
       b=dialog_save_changed_record(pane, record_changed);
-      if (b==DIALOG_SAID_1) {
+      if (b==DIALOG_SAID_2) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
 
@@ -826,7 +805,9 @@ static int memo_clear_details()
    }
 
    set_new_button_to(CLEAR_FLAG);
+   connect_changed_signals(CONNECT_SIGNALS);
    jp_logf(JP_LOG_DEBUG, "Leaving memo_clear_details()\n");
+
    return EXIT_SUCCESS;
 }
 
@@ -1019,20 +1000,27 @@ static void cb_clist_selection(GtkWidget      *clist,
    MyMemo *mmemo;
    int i, index, count, b;
    int sorted_position;
-   int keep;
+   unsigned int unique_id = 0;
 
-   if ((!event) && (clist_hack)) return;
+   if ((record_changed==MODIFY_FLAG) || (record_changed==NEW_FLAG)) {
+      mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), row);
+      if (mmemo!=NULL) {
+	 unique_id = mmemo->unique_id;
+      }
 
-   /* HACK, see clist hack explanation in memo_gui.c */
-   if (clist_hack) {
-      keep=record_changed;
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, column);
       b=dialog_save_changed_record(pane, record_changed);
-      if (b==DIALOG_SAID_1) {
+      if (b==DIALOG_SAID_2) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
       set_new_button_to(CLEAR_FLAG);
-      clist_select_row(GTK_CLIST(clist), row, column);
+
+      if (unique_id)
+      {
+	 glob_find_id = unique_id;
+         memo_find();
+      } else {
+	 clist_select_row(GTK_CLIST(clist), row, column);
+      }
       return;
    }
 
@@ -1050,9 +1038,7 @@ static void cb_clist_selection(GtkWidget      *clist,
       */
    {
       set_new_button_to(UNDELETE_FLAG);
-   }
-   else
-   {
+   } else {
       set_new_button_to(CLEAR_FLAG);
    }
 
@@ -1107,9 +1093,20 @@ static gboolean cb_key_pressed_left_side(GtkWidget   *widget,
                                          GdkEventKey *event,
                                          gpointer     next_widget)
 {
+#ifdef ENABLE_GTK2
+   GtkTextBuffer *text_buffer;
+   GtkTextIter    iter;
+#endif
+
    if (event->keyval == GDK_Return) {
       gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
       gtk_widget_grab_focus(GTK_WIDGET(next_widget));
+#ifdef ENABLE_GTK2
+      /* Position cursor at start of text */
+      text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(next_widget));
+      gtk_text_buffer_get_start_iter(text_buffer, &iter);
+      gtk_text_buffer_place_cursor(text_buffer, &iter);
+#endif
       return TRUE;
    }
 
@@ -1248,7 +1245,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    /* If there are items in the list, highlight the selected row */
    if ((main) && (entries_shown>0)) {
       /* Select the existing requested row, or row 0 if that is impossible */
-      if (clist_row_selected <= entries_shown) {
+      if (clist_row_selected < entries_shown) {
 	 clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
 	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), clist_row_selected)) {
 	    gtk_clist_moveto(GTK_CLIST(clist), clist_row_selected, 0, 0.5, 0.0);
@@ -1316,7 +1313,7 @@ int memo_cycle_cat()
    int i, new_cat;
 
    b=dialog_save_changed_record(pane, record_changed);
-   if (b==DIALOG_SAID_1) {
+   if (b==DIALOG_SAID_2) {
       cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
    }
 
@@ -1370,15 +1367,11 @@ int memo_gui_cleanup()
    int b;
 
    b=dialog_save_changed_record(pane, record_changed);
-   if (b==DIALOG_SAID_1) {
+   if (b==DIALOG_SAID_2) {
       cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
    }
    free_MemoList(&glob_memo_list);
    connect_changed_signals(DISCONNECT_SIGNALS);
-#ifdef ENABLE_GTK2
-   glob_gtk_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
-   glob_x11_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY));
-#endif
 #ifdef ENABLE_GTK2
    set_pref(PREF_MEMO_PANE, gtk_paned_get_position(GTK_PANED(pane)), NULL, TRUE);
 #else
@@ -1441,24 +1434,6 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
       memo_category=CATEGORY_ALL;
    }
 
-#ifdef ENABLE_GTK2
-   /* Preserve clipboard buffer across applications */
-   if (glob_gtk_clipboard_text != NULL)
-   {
-      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD),
-                             glob_gtk_clipboard_text, -1);
-      g_free(glob_gtk_clipboard_text);
-      glob_gtk_clipboard_text = NULL;
-   }
-   if (glob_x11_clipboard_text != NULL)
-   {
-      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
-                             glob_x11_clipboard_text, -1);
-      g_free(glob_x11_clipboard_text);
-      glob_x11_clipboard_text = NULL;
-   }
-#endif
-
    accel_group = gtk_accel_group_new();
    gtk_window_add_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(vbox)),
       accel_group);
@@ -1486,7 +1461,7 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    glob_date_label = gtk_label_new(" ");
    gtk_box_pack_start(GTK_BOX(vbox1), glob_date_label, FALSE, FALSE, 0);
    timeout_date(NULL);
-   glob_date_timer_tag = gtk_timeout_add(CLOCK_TICK, timeout_date, NULL);
+   glob_date_timer_tag = gtk_timeout_add(get_timeout_interval(), timeout_date, NULL);
 
 
    /* Separator */
@@ -1519,21 +1494,11 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(vbox1), scrolled_window, TRUE, TRUE, 0);
 
    clist = gtk_clist_new(1);
-   clist_hack=FALSE;
 
    gtk_signal_connect(GTK_OBJECT(clist), "select_row",
 		      GTK_SIGNAL_FUNC(cb_clist_selection), NULL);
+
    gtk_clist_set_shadow_type(GTK_CLIST(clist), SHADOW);
-   /*
-    * clist hack explanation:
-    * There is a strange hack here.
-    * If a clist is in browse mode and a callback is called on it and then
-    * a dialog window is opened (which we do if a record has changed, and we
-    * ask to save it) then XFree86 locks up.
-    * This was tested with XFree86 4.1.0 / GTK 1.2.10
-    *                  and XFree86 3.3.6 / GTK 1.2.8
-    *  So, we switch the modes of the clist before opening a dialog window.
-    */
    gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
    gtk_clist_set_column_width(GTK_CLIST(clist), 0, 50);
 
@@ -1542,7 +1507,7 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    /*
     * The right hand part of the main window follows:
     */
-   hbox_temp = gtk_hbox_new(FALSE, 0);
+   hbox_temp = gtk_hbox_new(FALSE, 3);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
 
