@@ -1,4 +1,7 @@
-/* plugins.c
+/* $Id: plugins.c,v 1.16 2005/05/08 15:57:31 judd Exp $ */
+
+/*******************************************************************************
+ * plugins.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #ifdef  ENABLE_PLUGINS
@@ -28,6 +31,7 @@
 #include "log.h"
 #include "plugins.h"
 #include <dlfcn.h>
+#include "i18n.h"
 
 GList *plugins = NULL;
 
@@ -65,8 +69,8 @@ void write_plugin_sync_file()
 }
 
 
-/* 
- * This is just a repeated subroutine to load_plugins not needing 
+/*
+ * This is just a repeated subroutine to load_plugins not needing
  * a name of its own.
  * Assumes dir has already been checked
  */
@@ -81,7 +85,7 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
    count = 0;
    for (i=0; (dirent = readdir(dir)); i++) {
       if (i>1000) {
-	 jp_logf(JP_LOG_WARN, "load_plugins_sub1(): infinite loop\n");
+	 jp_logf(JP_LOG_WARN, "load_plugins_sub1(): %s\n", _("infinite loop"));
 	 return 0;
       }
       /* If the filename has either of these extensions then plug it in */
@@ -101,7 +105,7 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
 	    }
 	    new_plugin = malloc(sizeof(struct plugin_s));
 	    if (!new_plugin) {
-	       jp_logf(JP_LOG_DEBUG, "load plugins(): Out of memory\n");
+	       jp_logf(JP_LOG_WARN, "load plugins(): %s\n", _("Out of memory"));
 	       return count;
 	    }
 	    memcpy(new_plugin, &temp_plugin, sizeof(struct plugin_s));
@@ -189,16 +193,16 @@ static int get_plugin_sync_bits()
 
    in=jp_open_home_file(EPN".plugins", "r");
    if (!in) {
-      return 0;
+      return EXIT_SUCCESS;
    }
-   for (i=0; (!feof(in)); i++) {   
+   for (i=0; (!feof(in)); i++) {
       if (i>1000) {
-	 jp_logf(JP_LOG_WARN, "load_plugins(): infinite loop\n");
+	 jp_logf(JP_LOG_WARN, "load_plugins(): %s\n", _("infinite loop"));
 	 fclose(in);
-	 return 0;
+	 return EXIT_FAILURE;
       }
       line[0]='\0';
-      Pc = fgets(line, 1000, in);
+      Pc = fgets(line, sizeof(line), in);
       if (!Pc) {
 	 break;
       }
@@ -206,11 +210,11 @@ static int get_plugin_sync_bits()
 	 line[strlen(line)-1]='\0';
       }
       if ((!strncmp(line, "Version", 7)) && (strcmp(line, "Version 1"))) {
-	 jp_logf(JP_LOG_WARN, "While reading "EPN".plugins line 1:[%s]\n", line);
-	 jp_logf(JP_LOG_WARN, "Wrong Version\n");
-	 jp_logf(JP_LOG_WARN, "Check preferences->conduits\n");
+	 jp_logf(JP_LOG_WARN, _("While reading %s%s line 1:[%s]\n"), EPN, ".plugins", line);
+	 jp_logf(JP_LOG_WARN, _("Wrong Version\n"));
+	 jp_logf(JP_LOG_WARN, _("Check preferences->conduits\n"));
 	 fclose(in);
-	 return 0;
+	 return EXIT_FAILURE;
       }
       if (i>0) {
 	 if (toupper(line[0])=='N') {
@@ -225,7 +229,7 @@ static int get_plugin_sync_bits()
       }
    }
    fclose(in);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 static int get_plugin_info(struct plugin_s *p, char *path)
@@ -264,9 +268,9 @@ static int get_plugin_info(struct plugin_s *p, char *path)
 
    h = dlopen(path, RTLD_LAZY);
    if (!h) {
-      jp_logf(JP_LOG_WARN, "open failed on plugin [%s]\n error [%s]\n", path,
+      jp_logf(JP_LOG_WARN, _("Open failed on plugin [%s]\n error [%s]\n"), path,
 		  dlerror());
-      return -1;
+      return EXIT_FAILURE;
    }
    jp_logf(JP_LOG_DEBUG, "opened plugin [%s]\n", path);
    p->handle=h;
@@ -276,26 +280,26 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    /* plugin_versionM */
 #if defined __OpenBSD__ && !defined __ELF__
 #define dlsym(x,y) dlsym(x, "_" y)
-#endif     
+#endif
    plugin_versionM = dlsym(h, "plugin_version");
    if (plugin_versionM==NULL)  {
       err = dlerror();
       jp_logf(JP_LOG_WARN, "plugin_version: [%s]\n", err);
-      jp_logf(JP_LOG_WARN, " plugin is invalid: [%s]\n", path);
+      jp_logf(JP_LOG_WARN, _(" plugin is invalid: [%s]\n"), path);
       dlclose(h);
       p->handle=NULL;
-      return -1;
+      return EXIT_FAILURE;
    }
    plugin_versionM(&major_version, &minor_version);
    version=major_version*1000+minor_version;
    if ((major_version <= 0) && (minor_version < 99)) {
-      jp_logf(JP_LOG_WARN, "Plugin:[%s]\n", path);
-      jp_logf(JP_LOG_WARN, "This plugin is version (%d.%d).\n",
+      jp_logf(JP_LOG_WARN, _("Plugin:[%s]\n"), path);
+      jp_logf(JP_LOG_WARN, _("This plugin is version (%d.%d).\n"),
 		  major_version, minor_version);
-      jp_logf(JP_LOG_WARN, "It is too old to work with this version of J-Pilot.\n");
+      jp_logf(JP_LOG_WARN, _("It is too old to work with this version of J-Pilot.\n"));
       dlclose(h);
       p->handle=NULL;
-      return -1;
+      return EXIT_FAILURE;
    }
    jp_logf(JP_LOG_DEBUG, "This plugin is version (%d.%d).\n",
 	       major_version, minor_version);
@@ -307,10 +311,10 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    if (p->plugin_get_name==NULL)  {
       err = dlerror();
       jp_logf(JP_LOG_WARN, "plugin_get_name: [%s]\n", err);
-      jp_logf(JP_LOG_WARN, " plugin is invalid: [%s]\n", path);
+      jp_logf(JP_LOG_WARN, _(" plugin is invalid: [%s]\n"), path);
       dlclose(h);
       p->handle=NULL;
-      return -1;
+      return EXIT_FAILURE;
    }
 
    if (p->plugin_get_name) {
@@ -401,7 +405,7 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    p->plugin_unpack_cai_from_ai = dlsym(h, "plugin_unpack_cai_from_ai");
    p->plugin_pack_cai_into_ai = dlsym(h, "plugin_pack_cai_into_ai");
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /* This will always return the first plugin list entry */
