@@ -1,4 +1,4 @@
-/* $Id: print.c,v 1.31 2005/10/24 15:27:48 rikster5 Exp $ */
+/* $Id: print.c,v 1.33 2006/05/26 20:52:14 judd Exp $ */
 
 /*******************************************************************************
  * print.c
@@ -108,7 +108,7 @@ int puttext(float x, float y, char *text)
    char *buf;
 
    len = strlen(text);
-   buf = (char *) malloc(2 * len + 1);
+   buf = malloc(2 * len + 1);
    memset(buf, 0, 2 * len + 1);
    ps_strncat(buf, text, 2 * len);
    fprintf(out, "%g inch %g inch moveto (%s) show\n", x, y, buf);
@@ -539,18 +539,15 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 
    /*------------------------------------------------------------------*/
    memcpy(&date, date_in, sizeof(struct tm));
-   sub_months_from_date(&date, 1);
    date.tm_mday = 1;	/* Go to the first of the month */
    mktime(&date);
+   sub_months_from_date(&date, 1);
    strftime(desc, sizeof(desc), "(%B %Y) %w ", &date);
    fprintf(out, "\n\n%%----------------------------------------\n"
            "%% Now generate the small months\n\n"
            "%s %d ", desc, days_in_mon(&date));
 
-   memcpy(&date, date_in, sizeof(struct tm));
-   add_months_to_date(&date, 1);
-   date.tm_mday = 1;	/* Go to the first of the month */
-   mktime(&date);
+   add_months_to_date(&date, 2);
    strftime(desc, sizeof(desc), "(%B %Y) %w ", &date);
    fprintf(out, "%s %d SmallMonths\n", desc, days_in_mon(&date));
 
@@ -1180,7 +1177,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
 	 char *buf;
 
 	 len = strlen(todo->description);
-	 buf = (char *) malloc(2 * len + 1);
+	 buf = malloc(2 * len + 1);
 	 memset(buf, 0, 2 * len + 1);
 	 ps_strncat(buf, todo->description, 2 * len);
 
@@ -1195,7 +1192,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
 	 char *buf;
 
 	 len = strlen(todo->note);
-	 buf = (char *) malloc(2 * len + 1);
+	 buf = malloc(2 * len + 1);
 	 memset(buf, 0, 2 * len + 1);
 	 ps_strncat(buf, todo->note, 2 * len);
 
