@@ -1,4 +1,4 @@
-/* $Id: todo_gui.c,v 1.93 2005/10/24 19:15:41 judd Exp $ */
+/* $Id: todo_gui.c,v 1.106 2006/06/25 03:36:36 rikster5 Exp $ */
 
 /*******************************************************************************
  * todo_gui.c
@@ -50,10 +50,6 @@
 #define DISCONNECT_SIGNALS 401
 
 extern GtkTooltips *glob_tooltips;
-#ifdef ENABLE_GTK2
-extern gchar *glob_gtk_clipboard_text;
-extern gchar *glob_x11_clipboard_text;
-#endif
 
 static GtkWidget *clist;
 static GtkWidget *todo_text, *todo_text_note;
@@ -90,7 +86,6 @@ int todo_category=CATEGORY_ALL;
 static int clist_col_selected;
 static int clist_row_selected;
 static int record_changed;
-static int clist_hack;
 
 void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 		       ToDoList **todo_list, int category, int main);
@@ -207,8 +202,7 @@ int todo_print()
    return EXIT_SUCCESS;
 }
 
-static void
-set_new_button_to(int new_state)
+static void set_new_button_to(int new_state)
 {
    jp_logf(JP_LOG_DEBUG, "set_new_button_to new %d old %d\n", new_state, record_changed);
 
@@ -218,72 +212,53 @@ set_new_button_to(int new_state)
 
    switch (new_state) {
     case MODIFY_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
-      clist_hack=TRUE;
-      /* The line selected on the clist becomes unhighlighted, so we do this */
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
-      gtk_widget_show(apply_record_button);
       gtk_widget_show(cancel_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(apply_record_button);
+
+      gtk_widget_hide(add_record_button);
       gtk_widget_hide(delete_record_button);
+      gtk_widget_hide(new_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case NEW_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
-      clist_hack=TRUE;
-      /* The line selected on the clist becomes unhighlighted, so we do this */
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
-      gtk_widget_show(add_record_button);
       gtk_widget_show(cancel_record_button);
+      gtk_widget_show(add_record_button);
+
+      gtk_widget_hide(apply_record_button);
       gtk_widget_hide(copy_record_button);
       gtk_widget_hide(delete_record_button);
+      gtk_widget_hide(new_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case CLEAR_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-      clist_hack=FALSE;
-      gtk_widget_show(new_record_button);
-      gtk_widget_show(copy_record_button);
       gtk_widget_show(delete_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(new_record_button);
+
+      gtk_widget_hide(add_record_button);
+      gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
+      gtk_widget_hide(undelete_record_button);
+
       break;
     case UNDELETE_FLAG:
-      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-      clist_hack=FALSE;
-      gtk_widget_hide(delete_record_button);
       gtk_widget_show(undelete_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(new_record_button);
+
+      gtk_widget_hide(add_record_button);
+      gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
+      gtk_widget_hide(delete_record_button);
       break;
+
     default:
       return;
    }
-   switch (record_changed) {
-    case MODIFY_FLAG:
-      gtk_widget_hide(apply_record_button);
-      gtk_widget_hide(cancel_record_button);
-      gtk_widget_show(copy_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-    case NEW_FLAG:
-      gtk_widget_hide(add_record_button);
-      gtk_widget_hide(cancel_record_button);
-      gtk_widget_show(copy_record_button);
-      gtk_widget_show(delete_record_button);
-      break;
-    case CLEAR_FLAG:
-      if (new_state != UNDELETE_FLAG)
-      {
-         gtk_widget_hide(new_record_button);
-         gtk_widget_hide(delete_record_button);
-      }
-      else
-      {
-	 gtk_widget_hide(delete_record_button);
-      }
-      break;
-    case UNDELETE_FLAG:
-      if (new_state != UNDELETE_FLAG)
-      {
-	 gtk_widget_hide(undelete_record_button);
-	 gtk_widget_show(delete_record_button);
-      }
-      break;
-   }
+
    record_changed=new_state;
 }
 
@@ -459,8 +434,7 @@ int todo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 #ifdef JPILOT_DEBUG
 	 printf("category is [%s]\n", text);
 #endif
-	 strncpy(old_cat_name, text, 16);
-	 old_cat_name[16]='\0';
+	 g_strlcpy(old_cat_name, text, 17);
 	 attrib=0;
 	 /* Figure out what the best category number is */
 	 suggested_cat_num=0;
@@ -560,7 +534,7 @@ int todo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
       if (dat_check_if_dat_file(in)!=DAT_TODO_FILE) {
 	 jp_logf(JP_LOG_WARN, _("File doesn't appear to be todo.dat format\n"));
 	 fclose(in);
-	 return 1;
+	 return EXIT_FAILURE;
       }
       todolist=NULL;
       dat_get_todos(in, &todolist, &cai);
@@ -568,12 +542,10 @@ int todo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
       for (temp_todolist=todolist; temp_todolist; temp_todolist=temp_todolist->next) {
 	 index=temp_todolist->mtodo.unique_id-1;
 	 if (index<0) {
-	    strncpy(old_cat_name, _("Unfiled"), 16);
-	    old_cat_name[16]='\0';
+	    g_strlcpy(old_cat_name, _("Unfiled"), 17);
 	    index=0;
 	 } else {
-	    strncpy(old_cat_name, cai.name[index], 16);
-	    old_cat_name[16]='\0';
+	    g_strlcpy(old_cat_name, cai.name[index], 17);
 	 }
 	 attrib=0;
 	 /* Figure out what category it was in the dat file */
@@ -659,7 +631,7 @@ void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    time_t ltime;
    struct tm *now;
    char *button_text[]={N_("OK")};
-   char *button_overwrite_text[]={N_("Yes"), N_("No")};
+   char *button_overwrite_text[]={N_("No"), N_("Yes")};
    char text[1024];
    char str1[256], str2[256];
    char pref_time[40];
@@ -673,8 +645,7 @@ void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    /* this stuff is for ical only. */
    /* todo: create a pre-export switch */
    get_pref(PREF_USER, NULL, &svalue);
-   strncpy(text, svalue, sizeof(text));
-   text[sizeof(text)-1]='\0';
+   g_strlcpy(text, svalue, sizeof(text));
    str_to_ical_str(username, sizeof(username), text);
    get_pref(PREF_USER_ID, &userid, NULL);
    gethostname(text, sizeof(text));
@@ -687,15 +658,15 @@ void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
       if (S_ISDIR(statb.st_mode)) {
 	 g_snprintf(text, sizeof(text), _("%s is a directory"), filename);
 	 dialog_generic(GTK_WINDOW(export_window),
-			0, 0, _("Error Opening File"),
-			_("Directory"), text, 1, button_text);
+			_("Error Opening File"),
+			DIALOG_ERROR, text, 1, button_text);
 	 return;
       }
       g_snprintf(text, sizeof(text), _("Do you want to overwrite file %s?"), filename);
       r = dialog_generic(GTK_WINDOW(export_window),
-			 0, 0, _("Overwrite File?"),
-			 _("Overwrite File"), text, 2, button_overwrite_text);
-      if (r!=DIALOG_SAID_1) {
+			 _("Overwrite File?"),
+			 DIALOG_QUESTION, text, 2, button_overwrite_text);
+      if (r!=DIALOG_SAID_2) {
 	 return;
       }
    }
@@ -704,8 +675,8 @@ void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    if (!out) {
       g_snprintf(text, sizeof(text), _("Error opening file: %s"), filename);
       dialog_generic(GTK_WINDOW(export_window),
-		     0, 0, _("Error Opening File"),
-		     _("Filename"), text, 1, button_text);
+		     _("Error Opening File"),
+		     DIALOG_ERROR, text, 1, button_text);
       return;
    }
 
@@ -1008,7 +979,7 @@ static void cb_category(GtkWidget *item, int selection)
 
    if ((GTK_CHECK_MENU_ITEM(item))->active) {
       b=dialog_save_changed_record(pane, record_changed);
-      if (b==DIALOG_SAID_1) {
+      if (b==DIALOG_SAID_2) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
 
@@ -1435,27 +1406,34 @@ static void cb_clist_selection(GtkWidget      *clist,
 			       GdkEventButton *event,
 			       gpointer       data)
 {
-   struct ToDo *todo;/*, new_a; */
+   struct ToDo *todo;
    MyToDo *mtodo;
-   int i, index, count;
+   int i, index, count, b;
    int sorted_position;
-   int keep, b;
+   unsigned int unique_id = 0;
 
    time_t ltime;
    struct tm *now;
 
-   if ((!event) && (clist_hack)) return;
+   if ((record_changed==MODIFY_FLAG) || (record_changed==NEW_FLAG)) {
+      mtodo = gtk_clist_get_row_data(GTK_CLIST(clist), row);
+      if (mtodo!=NULL) {
+	 unique_id = mtodo->unique_id;
+      }
 
-   /* HACK, see clist hack explanation in memo_gui.c */
-   if (clist_hack) {
-      keep=record_changed;
-      clist_select_row(GTK_CLIST(clist), clist_row_selected, column);
       b=dialog_save_changed_record(pane, record_changed);
-      if (b==DIALOG_SAID_1) {
+      if (b==DIALOG_SAID_2) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
       set_new_button_to(CLEAR_FLAG);
-      clist_select_row(GTK_CLIST(clist), row, column);
+
+      if (unique_id)
+      {
+	 glob_find_id = unique_id;
+         todo_find();
+      } else {
+	 clist_select_row(GTK_CLIST(clist), row, column);
+      }
       return;
    }
 
@@ -1476,9 +1454,7 @@ static void cb_clist_selection(GtkWidget      *clist,
       */
    {
       set_new_button_to(UNDELETE_FLAG);
-   }
-   else
-   {
+   } else {
       set_new_button_to(CLEAR_FLAG);
    }
 
@@ -1614,9 +1590,20 @@ static gboolean cb_key_pressed_left_side(GtkWidget   *widget,
                                          GdkEventKey *event,
                                          gpointer     next_widget)
 {
+#ifdef ENABLE_GTK2
+   GtkTextBuffer *text_buffer;
+   GtkTextIter    iter;
+#endif
+
    if (event->keyval == GDK_Return) {
       gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
       gtk_widget_grab_focus(GTK_WIDGET(next_widget));
+#ifdef ENABLE_GTK2
+      /* Position cursor at start of text */
+      text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(next_widget));
+      gtk_text_buffer_get_start_iter(text_buffer, &iter);
+      gtk_text_buffer_place_cursor(text_buffer, &iter);
+#endif
       return TRUE;
    }
 
@@ -1829,7 +1816,7 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    /* If there are items in the list, highlight the selected row */
    if ((main) && (entries_shown>0)) {
       /* Select the existing requested row, or row 0 if that is impossible */
-      if (clist_row_selected <= entries_shown)
+      if (clist_row_selected < entries_shown)
       {
 	 clist_select_row(GTK_CLIST(clist), clist_row_selected, TODO_PRIORITY_COLUMN);
 	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), clist_row_selected)) {
@@ -1919,7 +1906,7 @@ int todo_cycle_cat()
    int i, new_cat;
 
    b=dialog_save_changed_record(pane, record_changed);
-   if (b==DIALOG_SAID_1) {
+   if (b==DIALOG_SAID_2) {
       cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
    }
 
@@ -1973,15 +1960,11 @@ int todo_gui_cleanup()
    int b;
 
    b=dialog_save_changed_record(pane, record_changed);
-   if (b==DIALOG_SAID_1) {
+   if (b==DIALOG_SAID_2) {
       cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
    }
    free_ToDoList(&glob_todo_list);
    connect_changed_signals(DISCONNECT_SIGNALS);
-#ifdef ENABLE_GTK2
-   glob_gtk_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
-   glob_x11_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY));
-#endif
 #ifdef ENABLE_GTK2
    set_pref(PREF_TODO_PANE, gtk_paned_get_position(GTK_PANED(pane)), NULL, TRUE);
 #else
@@ -2049,24 +2032,6 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
       todo_category=CATEGORY_ALL;
    }
 
-#ifdef ENABLE_GTK2
-   /* Preserve clipboard buffer across applications */
-   if (glob_gtk_clipboard_text != NULL)
-   {
-      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD),
-                             glob_gtk_clipboard_text, -1);
-      g_free(glob_gtk_clipboard_text);
-      glob_gtk_clipboard_text = NULL;
-   }
-   if (glob_x11_clipboard_text != NULL)
-   {
-      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
-                             glob_x11_clipboard_text, -1);
-      g_free(glob_x11_clipboard_text);
-      glob_x11_clipboard_text = NULL;
-   }
-#endif
-
    accel_group = gtk_accel_group_new();
    gtk_window_add_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(vbox)),
       accel_group);
@@ -2097,7 +2062,7 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    glob_date_label = gtk_label_new(" ");
    gtk_box_pack_start(GTK_BOX(vbox1), glob_date_label, FALSE, FALSE, 0);
    timeout_date(NULL);
-   glob_date_timer_tag = gtk_timeout_add(CLOCK_TICK, timeout_date, NULL);
+   glob_date_timer_tag = gtk_timeout_add(get_timeout_interval(), timeout_date, NULL);
 
    /*Separator */
    separator = gtk_hseparator_new();
@@ -2127,7 +2092,7 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(vbox1), scrolled_window, TRUE, TRUE, 0);
 
    clist = gtk_clist_new_with_titles(5, titles);
-   clist_hack=FALSE;
+   
    gtk_clist_set_column_title(GTK_CLIST(clist), TODO_TEXT_COLUMN, _("Task"));
    gtk_clist_set_column_title(GTK_CLIST(clist), TODO_DATE_COLUMN, _("Due"));
    /* Put pretty pictures in the clist column headings */
@@ -2159,7 +2124,7 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    /* */
    /* The right hand part of the main window follows: */
    /* */
-   hbox_temp = gtk_hbox_new(FALSE, 0);
+   hbox_temp = gtk_hbox_new(FALSE, 3);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
 
